@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2010 Nokia Corporation
+ * @copyright Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,7 +27,7 @@ namespace Tp
 
 class ConnectionCapabilities;
 
-struct AccountSet::Private
+struct TELEPATHY_QT4_NO_EXPORT AccountSet::Private
 {
     class AccountWrapper;
 
@@ -49,7 +50,6 @@ struct AccountSet::Private
     AccountFilterConstPtr filter;
     QHash<QString, AccountWrapper *> wrappers;
     QHash<QString, AccountPtr> accounts;
-    bool filterValid;
     bool ready;
 };
 
@@ -71,9 +71,9 @@ Q_SIGNALS:
             const Tp::ConnectionCapabilities &capabilities);
 
 private Q_SLOTS:
-    void onAccountRemoved();
-    void onAccountPropertyChanged(const QString &propertyName);
-    void onAccountCapalitiesChanged(const Tp::ConnectionCapabilities &capabilities);
+    TELEPATHY_QT4_NO_EXPORT void onAccountRemoved();
+    TELEPATHY_QT4_NO_EXPORT void onAccountPropertyChanged(const QString &propertyName);
+    TELEPATHY_QT4_NO_EXPORT void onAccountCapalitiesChanged(const Tp::ConnectionCapabilities &capabilities);
 
 private:
     AccountPtr mAccount;
