@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2010 Nokia Corporation
+ * @copyright Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -63,9 +64,9 @@ Q_SIGNALS:
     void accountRemoved(const Tp::AccountPtr &account);
 
 private Q_SLOTS:
-    void onNewAccount(const Tp::AccountPtr &account);
-    void onAccountRemoved(const Tp::AccountPtr &account);
-    void onAccountChanged(const Tp::AccountPtr &account);
+    TELEPATHY_QT4_NO_EXPORT void onNewAccount(const Tp::AccountPtr &account);
+    TELEPATHY_QT4_NO_EXPORT void onAccountRemoved(const Tp::AccountPtr &account);
+    TELEPATHY_QT4_NO_EXPORT void onAccountChanged(const Tp::AccountPtr &account);
 
 private:
     struct Private;
