@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009-2010 Nokia Corporation
+ * @copyright Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,15 +33,18 @@
 #include <TelepathyQt4/ContactSearchChannel>
 #include <TelepathyQt4/FileTransferChannel>
 #include <TelepathyQt4/IncomingFileTransferChannel>
+#include <TelepathyQt4/IncomingStreamTubeChannel>
 #include <TelepathyQt4/OutgoingFileTransferChannel>
+#include <TelepathyQt4/OutgoingStreamTubeChannel>
 #include <TelepathyQt4/RoomListChannel>
+#include <TelepathyQt4/StreamTubeChannel>
 #include <TelepathyQt4/StreamedMediaChannel>
 #include <TelepathyQt4/TextChannel>
 
 namespace Tp
 {
 
-struct ChannelFactory::Private
+struct TELEPATHY_QT4_NO_EXPORT ChannelFactory::Private
 {
     Private();
 
@@ -98,6 +102,8 @@ ChannelFactory::ChannelFactory(const QDBusConnection &bus)
     setSubclassForRoomLists<RoomListChannel>();
     setSubclassForIncomingFileTransfers<IncomingFileTransferChannel>();
     setSubclassForOutgoingFileTransfers<OutgoingFileTransferChannel>();
+    setSubclassForIncomingStreamTubes<IncomingStreamTubeChannel>();
+    setSubclassForOutgoingStreamTubes<OutgoingStreamTubeChannel>();
     setSubclassForContactSearches<ContactSearchChannel>();
     setFallbackSubclass<Channel>();
 }
@@ -258,6 +264,52 @@ void ChannelFactory::setConstructorForIncomingFileTransfers(const ConstructorCon
     setConstructorFor(ChannelClassSpec::incomingFileTransfer(additionalProps), ctor);
 }
 
+Features ChannelFactory::featuresForOutgoingStreamTubes(const QVariantMap &additionalProps) const
+{
+    return featuresFor(ChannelClassSpec::outgoingStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::addFeaturesForOutgoingStreamTubes(const Features &features,
+        const QVariantMap &additionalProps)
+{
+    addFeaturesFor(ChannelClassSpec::outgoingStreamTube(QString(), additionalProps), features);
+}
+
+ChannelFactory::ConstructorConstPtr ChannelFactory::constructorForOutgoingStreamTubes(
+        const QVariantMap &additionalProps) const
+{
+    return constructorFor(ChannelClassSpec::outgoingStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::setConstructorForOutgoingStreamTubes(const ConstructorConstPtr &ctor,
+        const QVariantMap &additionalProps)
+{
+    setConstructorFor(ChannelClassSpec::outgoingStreamTube(QString(), additionalProps), ctor);
+}
+
+Features ChannelFactory::featuresForIncomingStreamTubes(const QVariantMap &additionalProps) const
+{
+    return featuresFor(ChannelClassSpec::incomingStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::addFeaturesForIncomingStreamTubes(const Features &features,
+        const QVariantMap &additionalProps)
+{
+    addFeaturesFor(ChannelClassSpec::incomingStreamTube(QString(), additionalProps), features);
+}
+
+ChannelFactory::ConstructorConstPtr ChannelFactory::constructorForIncomingStreamTubes(
+        const QVariantMap &additionalProps) const
+{
+    return constructorFor(ChannelClassSpec::incomingStreamTube(QString(), additionalProps));
+}
+
+void ChannelFactory::setConstructorForIncomingStreamTubes(const ConstructorConstPtr &ctor,
+        const QVariantMap &additionalProps)
+{
+    setConstructorFor(ChannelClassSpec::incomingStreamTube(QString(), additionalProps), ctor);
+}
+
 Features ChannelFactory::featuresForContactSearches(const QVariantMap &additionalProps) const
 {
     return featuresFor(ChannelClassSpec::contactSearch(additionalProps));
