@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009-2010 Nokia Corporation
+ * @copyright Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -157,6 +158,38 @@ public:
     void setConstructorForIncomingFileTransfers(const ConstructorConstPtr &ctor,
             const QVariantMap &additionalProps = QVariantMap());
 
+    Features featuresForOutgoingStreamTubes(const QVariantMap &additionalProps = QVariantMap()) const;
+    void addFeaturesForOutgoingStreamTubes(const Features &features,
+            const QVariantMap &additionalProps = QVariantMap());
+
+    ConstructorConstPtr constructorForOutgoingStreamTubes(
+            const QVariantMap &additionalProps = QVariantMap()) const;
+
+    template<typename Subclass>
+    void setSubclassForOutgoingStreamTubes(const QVariantMap &additionalProps = QVariantMap())
+    {
+        setConstructorForOutgoingStreamTubes(SubclassCtor<Subclass>::create(), additionalProps);
+    }
+
+    void setConstructorForOutgoingStreamTubes(const ConstructorConstPtr &ctor,
+            const QVariantMap &additionalProps = QVariantMap());
+
+    Features featuresForIncomingStreamTubes(const QVariantMap &additionalProps = QVariantMap()) const;
+    void addFeaturesForIncomingStreamTubes(const Features &features,
+            const QVariantMap &additionalProps = QVariantMap());
+
+    ConstructorConstPtr constructorForIncomingStreamTubes(
+            const QVariantMap &additionalProps = QVariantMap()) const;
+
+    template<typename Subclass>
+    void setSubclassForIncomingStreamTubes(const QVariantMap &additionalProps = QVariantMap())
+    {
+        setConstructorForIncomingStreamTubes(SubclassCtor<Subclass>::create(), additionalProps);
+    }
+
+    void setConstructorForIncomingStreamTubes(const ConstructorConstPtr &ctor,
+            const QVariantMap &additionalProps = QVariantMap());
+
     Features featuresForContactSearches(const QVariantMap &additionalProps = QVariantMap()) const;
     void addFeaturesForContactSearches(const Features &features,
             const QVariantMap &additionalProps = QVariantMap());
@@ -173,8 +206,6 @@ public:
     void setConstructorForContactSearches(const ConstructorConstPtr &ctor,
             const QVariantMap &additionalProps = QVariantMap());
 
-    // When merged, Tube channels should have export/import variants too like FT has for send/receive
-
     Features commonFeatures() const;
     void addCommonFeatures(const Features &features);
 
