@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,6 +38,7 @@
 #include <TelepathyQt4/Types>
 #include <TelepathyQt4/SharedPtr>
 
+#include <QSharedDataPointer>
 #include <QString>
 #include <QStringList>
 #include <QVariantMap>
@@ -44,6 +46,7 @@
 namespace Tp
 {
 
+class ChannelRequestHints;
 class PendingOperation;
 
 class TELEPATHY_QT4_EXPORT ChannelRequest : public StatefulDBusProxy,
@@ -71,14 +74,18 @@ public:
     QDateTime userActionTime() const;
     QString preferredHandler() const;
     QualifiedPropertyValueMapList requests() const;
+    ChannelRequestHints hints() const;
 
     QVariantMap immutableProperties() const;
 
     PendingOperation *cancel();
 
+    ChannelPtr channel() const;
+
 Q_SIGNALS:
     void failed(const QString &errorName, const QString &errorMessage);
-    void succeeded();
+    void succeeded(); // TODO API/ABI break: remove
+    void succeeded(const Tp::ChannelPtr &channel);
 
 protected:
     ChannelRequest(const QDBusConnection &bus,
@@ -93,9 +100,18 @@ protected:
 
     Client::ChannelRequestInterface *baseInterface() const;
 
+protected:
+    // TODO: (API/ABI break) Remove connectNotify
+    void connectNotify(const char *);
+
 private Q_SLOTS:
-    void gotMainProperties(QDBusPendingCallWatcher *watcher);
-    void onAccountReady(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void gotMainProperties(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onAccountReady(Tp::PendingOperation *op);
+
+    TELEPATHY_QT4_NO_EXPORT void onLegacySucceeded();
+    TELEPATHY_QT4_NO_EXPORT void onSucceededWithChannel(const QDBusObjectPath &connPath, const QVariantMap &connProps,
+            const QDBusObjectPath &chanPath, const QVariantMap &chanProps);
+    TELEPATHY_QT4_NO_EXPORT void onChanBuilt(Tp::PendingOperation *op);
 
 private:
     friend class PendingChannelRequest;
@@ -107,6 +123,32 @@ private:
     Private *mPriv;
 };
 
+class TELEPATHY_QT4_EXPORT ChannelRequestHints
+{
+public:
+    ChannelRequestHints();
+    ChannelRequestHints(const QVariantMap &hints);
+    ChannelRequestHints(const ChannelRequestHints &other);
+    ~ChannelRequestHints();
+
+    ChannelRequestHints &operator=(const ChannelRequestHints &other);
+
+    bool isValid() const;
+
+    bool hasHint(const QString &reversedDomain, const QString &localName) const;
+    QVariant hint(const QString &reversedDomain, const QString &localName) const;
+    void setHint(const QString &reversedDomain, const QString &localName, const QVariant &value);
+
+    QVariantMap allHints() const;
+
+private:
+    struct Private;
+    friend struct Private;
+    QSharedDataPointer<Private> mPriv;
+};
+
 } // Tp
 
+Q_DECLARE_METATYPE(Tp::ChannelRequestHints);
+
 #endif
