@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * @copyright Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -136,7 +137,7 @@ public:
         friend class Contact;
         friend class ContactManager;
 
-        GroupMemberChangeDetails(const ContactPtr &actor, const QVariantMap &details);
+        TELEPATHY_QT4_NO_EXPORT GroupMemberChangeDetails(const ContactPtr &actor, const QVariantMap &details);
 
         struct Private;
         friend struct Private;
@@ -200,42 +201,42 @@ protected Q_SLOTS:
     PendingOperation *groupAddSelfHandle();
 
 private Q_SLOTS:
-    void gotMainProperties(QDBusPendingCallWatcher *watcher);
-    void gotChannelType(QDBusPendingCallWatcher *watcher);
-    void gotHandle(QDBusPendingCallWatcher *watcher);
-    void gotInterfaces(QDBusPendingCallWatcher *watcher);
-    void onClosed();
+    TELEPATHY_QT4_NO_EXPORT void gotMainProperties(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotChannelType(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotHandle(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotInterfaces(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onClosed();
 
-    void onConnectionReady(Tp::PendingOperation *op);
-    void onConnectionInvalidated();
+    TELEPATHY_QT4_NO_EXPORT void onConnectionReady(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onConnectionInvalidated();
 
-    void gotGroupProperties(QDBusPendingCallWatcher *watcher);
-    void gotGroupFlags(QDBusPendingCallWatcher *watcher);
-    void gotAllMembers(QDBusPendingCallWatcher *watcher);
-    void gotLocalPendingMembersWithInfo(QDBusPendingCallWatcher *watcher);
-    void gotSelfHandle(QDBusPendingCallWatcher *watcher);
-    void gotContacts(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void gotGroupProperties(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotGroupFlags(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotAllMembers(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotLocalPendingMembersWithInfo(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotSelfHandle(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotContacts(Tp::PendingOperation *op);
 
-    void onGroupFlagsChanged(uint added, uint removed);
-    void onMembersChanged(const QString &message,
+    TELEPATHY_QT4_NO_EXPORT void onGroupFlagsChanged(uint added, uint removed);
+    TELEPATHY_QT4_NO_EXPORT void onMembersChanged(const QString &message,
             const Tp::UIntList &added, const Tp::UIntList &removed,
             const Tp::UIntList &localPending, const Tp::UIntList &remotePending,
             uint actor, uint reason);
-    void onMembersChangedDetailed(
+    TELEPATHY_QT4_NO_EXPORT void onMembersChangedDetailed(
             const Tp::UIntList &added, const Tp::UIntList &removed,
             const Tp::UIntList &localPending, const Tp::UIntList &remotePending,
             const QVariantMap &details);
-    void onHandleOwnersChanged(const Tp::HandleOwnerMap &added, const Tp::UIntList &removed);
-    void onSelfHandleChanged(uint selfHandle);
+    TELEPATHY_QT4_NO_EXPORT void onHandleOwnersChanged(const Tp::HandleOwnerMap &added, const Tp::UIntList &removed);
+    TELEPATHY_QT4_NO_EXPORT void onSelfHandleChanged(uint selfHandle);
 
-    void gotConferenceProperties(QDBusPendingCallWatcher *watcher);
-    void gotConferenceInitialInviteeContacts(Tp::PendingOperation *op);
-    void onConferenceChannelMerged(const QDBusObjectPath &channel, uint channelSpecificHandle,
+    TELEPATHY_QT4_NO_EXPORT void gotConferenceProperties(QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void gotConferenceInitialInviteeContacts(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onConferenceChannelMerged(const QDBusObjectPath &channel, uint channelSpecificHandle,
             const QVariantMap &properties);
-    void onConferenceChannelMerged(const QDBusObjectPath &channel);
-    void onConferenceChannelRemoved(const QDBusObjectPath &channel, const QVariantMap &details);
-    void onConferenceChannelRemoved(const QDBusObjectPath &channel);
-    void gotConferenceChannelRemovedActorContact(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onConferenceChannelMerged(const QDBusObjectPath &channel);
+    TELEPATHY_QT4_NO_EXPORT void onConferenceChannelRemoved(const QDBusObjectPath &channel, const QVariantMap &details);
+    TELEPATHY_QT4_NO_EXPORT void onConferenceChannelRemoved(const QDBusObjectPath &channel);
+    TELEPATHY_QT4_NO_EXPORT void gotConferenceChannelRemovedActorContact(Tp::PendingOperation *op);
 
 private:
     class PendingLeave;
