@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -106,10 +107,10 @@ protected:
     Client::ConnectionManagerInterface *baseInterface() const;
 
 private Q_SLOTS:
-    void gotMainProperties(QDBusPendingCallWatcher *);
-    void gotProtocolsLegacy(QDBusPendingCallWatcher *);
-    void gotParametersLegacy(QDBusPendingCallWatcher *);
-    void onProtocolReady(Tp::PendingOperation *);
+    TELEPATHY_QT4_NO_EXPORT void gotMainProperties(QDBusPendingCallWatcher *);
+    TELEPATHY_QT4_NO_EXPORT void gotProtocolsLegacy(QDBusPendingCallWatcher *);
+    TELEPATHY_QT4_NO_EXPORT void gotParametersLegacy(QDBusPendingCallWatcher *);
+    TELEPATHY_QT4_NO_EXPORT void onProtocolReady(Tp::PendingOperation *);
 
 private:
     friend class PendingConnection;
