@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -79,6 +80,7 @@ struct TELEPATHY_QT4_NO_EXPORT Connection::Private
     static void introspectRoster(Private *self);
     static void introspectRosterGroups(Private *self);
     static void introspectBalance(Private *self);
+    static void introspectConnected(Private *self);
 
     void continueMainIntrospection();
     void setCurrentStatus(uint status);
@@ -160,6 +162,7 @@ struct TELEPATHY_QT4_NO_EXPORT ConnectionLowlevel::Private
     }
 
     QWeakPointer<Connection> conn;
+    HandleIdentifierMap contactsIds;
 };
 
 // Handle tracking
@@ -271,6 +274,17 @@ Connection::Private::Private(Connection *parent,
         this);
     introspectables[FeatureAccountBalance] = introspectableBalance;
 
+    ReadinessHelper::Introspectable introspectableConnected(
+        QSet<uint>() << (uint) -1 <<
+                        ConnectionStatusDisconnected <<
+                        ConnectionStatusConnecting <<
+                        ConnectionStatusConnected,                                                  // makesSenseForStatuses
+        Features() << FeatureCore,                                                                  // dependsOnFeatures (none)
+        QStringList(),                                                                              // dependsOnInterfaces (none)
+        (ReadinessHelper::IntrospectFunc) &Private::introspectConnected,
+        this);
+    introspectables[FeatureConnected] = introspectableConnected;
+
     readinessHelper->addIntrospectables(introspectables);
     readinessHelper->setCurrentStatus(status);
     parent->connect(readinessHelper,
@@ -528,6 +542,13 @@ void Connection::Private::introspectBalance(Connection::Private *self)
             SLOT(gotBalance(QDBusPendingCallWatcher*)));
 }
 
+void Connection::Private::introspectConnected(Connection::Private *self)
+{
+    if (self->pendingStatus == ConnectionStatusConnected) {
+        self->readinessHelper->setIntrospectCompleted(FeatureConnected, true);
+    }
+}
+
 void Connection::Private::continueMainIntrospection()
 {
     if (introspectMainQueue.isEmpty()) {
@@ -883,6 +904,16 @@ const Feature Connection::FeatureRosterGroups = Feature(QLatin1String(Connection
  */
 const Feature Connection::FeatureAccountBalance = Feature(QLatin1String(Connection::staticMetaObject.className()), 6);
 
+/**
+ * When this feature is prepared, it means that the connection status() is
+ * ConnectionStatusConnected.
+ *
+ * Note that if ConnectionFactory is being used with FeatureConnected set, Connection objects will
+ * only be signalled by the library when the corresponding connection is in status()
+ * ConnectionStatusConnected.
+ */
+const Feature Connection::FeatureConnected = Feature(QLatin1String(Connection::staticMetaObject.className()), 7);
+
 /**
  * Create a new connection object using QDBusConnection::sessionBus().
  *
@@ -1033,7 +1064,7 @@ ConnectionStatusReason Connection::statusReason() const
     return (ConnectionStatusReason) mPriv->statusReason;
 }
 
-struct Connection::ErrorDetails::Private : public QSharedData
+struct TELEPATHY_QT4_NO_EXPORT Connection::ErrorDetails::Private : public QSharedData
 {
     Private(const QVariantMap &details)
         : details(details) {}
@@ -2082,6 +2113,47 @@ QStringList ConnectionLowlevel::contactAttributeInterfaces() const
     return conn->mPriv->contactAttributeInterfaces;
 }
 
+void ConnectionLowlevel::injectContactIds(const HandleIdentifierMap &contactIds)
+{
+    if (!hasImmortalHandles()) {
+        return;
+    }
+
+    for (HandleIdentifierMap::const_iterator i = contactIds.constBegin();
+            i != contactIds.constEnd(); ++i) {
+        uint handle = i.key();
+        QString id = i.value();
+
+        if (!id.isEmpty()) {
+            QString currentId = mPriv->contactsIds.value(handle);
+
+            if (!currentId.isEmpty() && id != currentId) {
+                warning() << "Trying to overwrite contact id from" << currentId << "to" << id
+                    << "for the same handle" << handle << ", ignoring";
+            } else {
+                mPriv->contactsIds.insert(handle, id);
+            }
+        }
+    }
+}
+
+void ConnectionLowlevel::injectContactId(uint handle, const QString &contactId)
+{
+    HandleIdentifierMap contactIds;
+    contactIds.insert(handle, contactId);
+    injectContactIds(contactIds);
+}
+
+bool ConnectionLowlevel::hasContactId(uint handle) const
+{
+    return mPriv->contactsIds.contains(handle);
+}
+
+QString ConnectionLowlevel::contactId(uint handle) const
+{
+    return mPriv->contactsIds.value(handle);
+}
+
 /**
  * Return whether the handles last for the whole lifetime of the connection.
  *
