@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009-2010 Nokia Corporation
+ * @copyright Copyright (C) 2009-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,17 +39,20 @@ public:
     ContactCapabilities();
     virtual ~ContactCapabilities();
 
+    bool streamTubes(const QString &service) const;
+    QStringList streamTubeServices() const;
+
     // later:
-    // QStringList streamTubeServices() const;
+    // bool dbusTubes(const QString &service) const;
     // QStringList dbusTubeServices() const;
 
 private:
     friend class Contact;
 
-    ContactCapabilities(bool specificToContact);
-    ContactCapabilities(const RequestableChannelClassList &rccs,
+    TELEPATHY_QT4_NO_EXPORT ContactCapabilities(bool specificToContact);
+    TELEPATHY_QT4_NO_EXPORT ContactCapabilities(const RequestableChannelClassList &rccs,
             bool specificToContact);
-    ContactCapabilities(const RequestableChannelClassSpecList &rccSpecs,
+    TELEPATHY_QT4_NO_EXPORT ContactCapabilities(const RequestableChannelClassSpecList &rccSpecs,
             bool specificToContact);
 };
 
