@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,6 +46,8 @@ public:
     Roster(ContactManager *manager);
     virtual ~Roster();
 
+    ContactListState state() const;
+
     PendingOperation *introspect();
     PendingOperation *introspectGroups();
     void reset();
@@ -90,8 +93,11 @@ private Q_SLOTS:
     void gotContactListProperties(Tp::PendingOperation *op);
     void gotContactListContacts(QDBusPendingCallWatcher *watcher);
     void onContactListStateChanged(uint state);
+    void onContactListContactsChangedWithId(const Tp::ContactSubscriptionMap &changes,
+            const Tp::HandleIdentifierMap &ids, const Tp::HandleIdentifierMap &removals);
     void onContactListContactsChanged(const Tp::ContactSubscriptionMap &changes,
             const Tp::UIntList &removals);
+    void setStateSuccess();
 
     void onContactListNewContactsConstructed(Tp::PendingOperation *op);
     void onContactListGroupsChanged(const Tp::UIntList &contacts,
@@ -184,10 +190,12 @@ private:
 
     PendingOperation *introspectPendingOp;
     PendingOperation *introspectGroupsPendingOp;
+    uint pendingContactListState;
     uint contactListState;
     bool canChangeContactList;
     bool contactListRequestUsesMessage;
     bool gotContactListInitialContacts;
+    bool gotContactListContactsChangedWithId;
     bool groupsReintrospectionRequired;
     QSet<QString> cachedAllKnownGroups;
     bool contactListGroupPropertiesReceived;
@@ -248,14 +256,17 @@ struct TELEPATHY_QT4_NO_EXPORT ContactManager::Roster::ChannelInfo
 
 struct TELEPATHY_QT4_NO_EXPORT ContactManager::Roster::UpdateInfo
 {
-    UpdateInfo(const ContactSubscriptionMap &changes, const UIntList &removals)
+    UpdateInfo(const ContactSubscriptionMap &changes, const HandleIdentifierMap &ids,
+            const HandleIdentifierMap &removals)
         : changes(changes),
+          ids(ids),
           removals(removals)
     {
     }
 
     ContactSubscriptionMap changes;
-    UIntList removals;
+    HandleIdentifierMap ids;
+    HandleIdentifierMap removals;
 };
 
 struct TELEPATHY_QT4_NO_EXPORT ContactManager::Roster::GroupsUpdateInfo
