@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,10 +46,12 @@ ContactManager::Roster::Roster(ContactManager *contactManager)
       usingFallbackContactList(false),
       introspectPendingOp(0),
       introspectGroupsPendingOp(0),
+      pendingContactListState((uint) -1),
       contactListState((uint) -1),
       canChangeContactList(false),
       contactListRequestUsesMessage(false),
       gotContactListInitialContacts(false),
+      gotContactListContactsChangedWithId(false),
       groupsReintrospectionRequired(false),
       contactListGroupPropertiesReceived(false),
       processingContactListChanges(false),
@@ -61,6 +64,11 @@ ContactManager::Roster::~Roster()
 {
 }
 
+ContactListState ContactManager::Roster::state() const
+{
+    return (Tp::ContactListState) contactListState;
+}
+
 PendingOperation *ContactManager::Roster::introspect()
 {
     ConnectionPtr conn(contactManager->connection());
@@ -653,7 +661,7 @@ void ContactManager::Roster::gotContactListProperties(PendingOperation *op)
     contactListRequestUsesMessage = qdbus_cast<uint>(props[QLatin1String("RequestUsesMessage")]);
 
     // only update the status if we did not get it from ContactListStateChanged
-    if (contactListState == (uint) -1) {
+    if (pendingContactListState == (uint) -1) {
         uint state = qdbus_cast<uint>(props[QLatin1String("ContactListState")]);
         onContactListStateChanged(state);
     }
@@ -664,6 +672,12 @@ void ContactManager::Roster::gotContactListContacts(QDBusPendingCallWatcher *wat
     QDBusPendingReply<ContactAttributesMap> reply = *watcher;
 
     if (watcher->isError()) {
+        warning() << "Failed introspecting ContactList contacts";
+
+        contactListState = ContactListStateFailure;
+        debug() << "Setting state to failure";
+        emit contactManager->stateChanged((Tp::ContactListState) contactListState);
+
         // We may have been in state Failure and then Success, and FeatureRoster is already ready
         if (introspectPendingOp) {
             introspectPendingOp->setFinishedWithError(
@@ -693,8 +707,15 @@ void ContactManager::Roster::gotContactListContacts(QDBusPendingCallWatcher *wat
 
     // We may have been in state Failure and then Success, and FeatureRoster is already ready
     if (introspectPendingOp) {
+        // Will emit stateChanged() signal when the op is finished in idle
+        // callback. This is to ensure FeatureRoster is marked ready.
+        connect(introspectPendingOp,
+                SIGNAL(finished(Tp::PendingOperation *)),
+                SLOT(setStateSuccess()));
         introspectPendingOp->setFinished();
         introspectPendingOp = 0;
+    } else {
+        setStateSuccess();
     }
 
     if (groupsReintrospectionRequired) {
@@ -702,18 +723,38 @@ void ContactManager::Roster::gotContactListContacts(QDBusPendingCallWatcher *wat
     }
 }
 
+void ContactManager::Roster::setStateSuccess()
+{
+    if (contactManager->connection()->isValid()) {
+        debug() << "State is now success";
+        contactListState = ContactListStateSuccess;
+        emit contactManager->stateChanged((Tp::ContactListState) contactListState);
+    }
+}
+
 void ContactManager::Roster::onContactListStateChanged(uint state)
 {
-    if (contactListState == state) {
+    if (pendingContactListState == state) {
         // ignore redundant state changes
         return;
     }
 
+    pendingContactListState = state;
+
+    if (state == ContactListStateSuccess) {
+        introspectContactListContacts();
+        return;
+    }
+
     contactListState = state;
 
-    if (state == ContactListStateSuccess) {
-        introspectContactListContacts();
-    } else if (state == ContactListStateFailure) {
+    if (state == ContactListStateFailure) {
+        debug() << "State changed to failure, finishing roster introspection";
+    }
+
+    emit contactManager->stateChanged((Tp::ContactListState) state);
+
+    if (state == ContactListStateFailure) {
         // Consider it done here as the state may go from Failure to Success afterwards, in which
         // case the contacts will appear.
         Q_ASSERT(introspectPendingOp);
@@ -722,9 +763,34 @@ void ContactManager::Roster::onContactListStateChanged(uint state)
     }
 }
 
+void ContactManager::Roster::onContactListContactsChangedWithId(const Tp::ContactSubscriptionMap &changes,
+        const Tp::HandleIdentifierMap &ids, const Tp::HandleIdentifierMap &removals)
+{
+    debug() << "Got ContactList.ContactsChangedWithID with" << changes.size() <<
+        "changes and" << removals.size() << "removals";
+
+    gotContactListContactsChangedWithId = true;
+
+    if (!gotContactListInitialContacts) {
+        debug() << "Ignoring ContactList changes until initial contacts are retrieved";
+        return;
+    }
+
+    ConnectionPtr conn(contactManager->connection());
+    conn->lowlevel()->injectContactIds(ids);
+
+    contactListUpdatesQueue.enqueue(UpdateInfo(changes, ids, removals));
+    contactListChangesQueue.enqueue(&ContactManager::Roster::processContactListUpdates);
+    processContactListChanges();
+}
+
 void ContactManager::Roster::onContactListContactsChanged(const Tp::ContactSubscriptionMap &changes,
         const Tp::UIntList &removals)
 {
+    if (gotContactListContactsChangedWithId) {
+        return;
+    }
+
     debug() << "Got ContactList.ContactsChanged with" << changes.size() <<
         "changes and" << removals.size() << "removals";
 
@@ -733,7 +799,12 @@ void ContactManager::Roster::onContactListContactsChanged(const Tp::ContactSubsc
         return;
     }
 
-    contactListUpdatesQueue.enqueue(UpdateInfo(changes, removals));
+    HandleIdentifierMap removalsMap;
+    foreach (uint handle, removals) {
+        removalsMap.insert(handle, QString());
+    }
+
+    contactListUpdatesQueue.enqueue(UpdateInfo(changes, HandleIdentifierMap(), removalsMap));
     contactListChangesQueue.enqueue(&ContactManager::Roster::processContactListUpdates);
     processContactListChanges();
 }
@@ -797,7 +868,7 @@ void ContactManager::Roster::onContactListNewContactsConstructed(Tp::PendingOper
         emit contactManager->presencePublicationRequested(publishRequested);
     }
 
-    foreach (uint bareHandle, info.removals) {
+    foreach (uint bareHandle, info.removals.keys()) {
         ContactPtr contact = contactManager->lookupContactByHandle(bareHandle);
         if (!contact) {
             warning() << "Unable to find removed contact with handle" << bareHandle;
@@ -810,8 +881,6 @@ void ContactManager::Roster::onContactListNewContactsConstructed(Tp::PendingOper
             continue;
         }
 
-        contact->setSubscriptionState(SubscriptionStateNo);
-        contact->setPublishState(SubscriptionStateNo);
         cachedAllKnownContacts.remove(contact);
         removed << contact;
     }
@@ -821,6 +890,11 @@ void ContactManager::Roster::onContactListNewContactsConstructed(Tp::PendingOper
                 Channel::GroupMemberChangeDetails());
     }
 
+    foreach (const Tp::ContactPtr &contact, removed) {
+        contact->setSubscriptionState(SubscriptionStateNo);
+        contact->setPublishState(SubscriptionStateNo);
+    }
+
     processingContactListChanges = false;
     processContactListChanges();
 }
@@ -985,6 +1059,18 @@ void ContactManager::Roster::onContactListChannelReady()
 
         introspectContactList();
     } else if (++contactListChannelsReady == ChannelInfo::LastType) {
+        if (contactListChannels.isEmpty()) {
+            contactListState = ContactListStateFailure;
+            debug() << "State is failure, roster not supported";
+            emit contactManager->stateChanged((Tp::ContactListState) contactListState);
+
+            Q_ASSERT(introspectPendingOp);
+            introspectPendingOp->setFinishedWithError(TP_QT4_ERROR_NOT_IMPLEMENTED,
+                    QLatin1String("Roster not supported"));
+            introspectPendingOp = 0;
+            return;
+        }
+
         setContactListChannelsReady();
 
         updateContactsBlockState();
@@ -1003,6 +1089,11 @@ void ContactManager::Roster::onContactListChannelReady()
         updateContactsPresenceState();
 
         Q_ASSERT(introspectPendingOp);
+        // Will emit stateChanged() signal when the op is finished in idle
+        // callback. This is to ensure FeatureRoster is marked ready.
+        connect(introspectPendingOp,
+                SIGNAL(finished(Tp::PendingOperation *)),
+                SLOT(setStateSuccess()));
         introspectPendingOp->setFinished();
         introspectPendingOp = 0;
     }
@@ -1303,6 +1394,9 @@ void ContactManager::Roster::introspectContactList()
     connect(iface,
             SIGNAL(ContactListStateChanged(uint)),
             SLOT(onContactListStateChanged(uint)));
+    connect(iface,
+            SIGNAL(ContactsChangedWithID(Tp::ContactSubscriptionMap,Tp::HandleIdentifierMap,Tp::HandleIdentifierMap)),
+            SLOT(onContactListContactsChangedWithId(Tp::ContactSubscriptionMap,Tp::HandleIdentifierMap,Tp::HandleIdentifierMap)));
     connect(iface,
             SIGNAL(ContactsChanged(Tp::ContactSubscriptionMap,Tp::UIntList)),
             SLOT(onContactListContactsChanged(Tp::ContactSubscriptionMap,Tp::UIntList)));
