@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,7 +72,8 @@ struct TELEPATHY_QT4_NO_EXPORT ContactManager::Private
 ContactManager::Private::Private(ContactManager *parent, Connection *connection)
     : parent(parent),
       connection(connection),
-      roster(new ContactManager::Roster(parent))
+      roster(new ContactManager::Roster(parent)),
+      requestAvatarsIdle(false)
 {
 }
 
@@ -162,6 +164,17 @@ Features ContactManager::supportedFeatures() const
     return mPriv->supportedFeatures;
 }
 
+/**
+ * Return the progress made in retrieving the contact list.
+ *
+ * Change notification is via stateChanged().
+ * \return The contact list state.
+ */
+ContactListState ContactManager::state() const
+{
+    return mPriv->roster->state();
+}
+
 /**
  * Return a list of relevant contacts (a reasonable guess as to what should
  * be displayed as "the contact list").
@@ -183,6 +196,7 @@ Features ContactManager::supportedFeatures() const
 Contacts ContactManager::allKnownContacts() const
 {
     if (!connection()->isReady(Connection::FeatureRoster)) {
+        warning() << "Calling allKnownContacts() before FeatureRoster is ready";
         return Contacts();
     }
 
@@ -794,6 +808,19 @@ PendingContacts *ContactManager::contactsForHandles(const UIntList &handles,
                 QLatin1String("Connection::FeatureCore is not ready"));
     }
 
+    ConnectionLowlevelPtr connLowlevel = connection()->lowlevel();
+
+    if (connLowlevel->hasImmortalHandles() && realFeatures.isEmpty()) {
+        // try to avoid a roundtrip if all handles have an id set and no feature was requested
+        foreach (uint handle, handles) {
+            if (connLowlevel->hasContactId(handle)) {
+                ContactPtr contact = ensureContact(handle,
+                        connLowlevel->contactId(handle), realFeatures);
+                satisfyingContacts.insert(handle, contact);
+            }
+        }
+    }
+
     foreach (uint handle, handles) {
         ContactPtr contact = lookupContactByHandle(handle);
         if (contact) {
@@ -835,6 +862,14 @@ PendingContacts *ContactManager::contactsForHandles(const ReferencedHandles &han
     return contactsForHandles(handles.toList(), features);
 }
 
+PendingContacts *ContactManager::contactsForHandles(const HandleIdentifierMap &handles,
+        const Features &features)
+{
+    connection()->lowlevel()->injectContactIds(handles);
+
+    return contactsForHandles(handles.keys(), features);
+}
+
 PendingContacts *ContactManager::contactsForIdentifiers(const QStringList &identifiers,
         const Features &features)
 {
@@ -976,13 +1011,13 @@ void ContactManager::onAvatarRetrieved(uint handle, const QString &token,
         debug() << "Filename:" << avatarFileName;
         debug() << "MimeType:" << mimeType;
 
-        QTemporaryFile mimeTypeFile(QLatin1String("tpqt4-avatar-mimetype."));
+        QTemporaryFile mimeTypeFile(mimeTypeFileName);
         mimeTypeFile.open();
         mimeTypeFile.write(mimeType.toLatin1());
         mimeTypeFile.setAutoRemove(false);
         mimeTypeFile.rename(mimeTypeFileName);
 
-        QTemporaryFile avatarFile(QLatin1String("tpqt4-avatar-data."));
+        QTemporaryFile avatarFile(avatarFileName);
         avatarFile.open();
         avatarFile.write(data);
         avatarFile.setAutoRemove(false);
@@ -1053,8 +1088,28 @@ ContactPtr ContactManager::ensureContact(const ReferencedHandles &handle,
     if (!contact) {
         contact = connection()->contactFactory()->construct(this, handle, features, attributes);
         mPriv->contacts.insert(bareHandle, contact.data());
-    } else {
-        contact->augment(features, attributes);
+    }
+
+    contact->augment(features, attributes);
+
+    return contact;
+}
+
+ContactPtr ContactManager::ensureContact(uint bareHandle, const QString &id,
+        const Features &features)
+{
+    ContactPtr contact = lookupContactByHandle(bareHandle);
+
+    if (!contact) {
+        QVariantMap attributes;
+        attributes.insert(QLatin1String(TELEPATHY_INTERFACE_CONNECTION "/contact-id"), id);
+
+        contact = connection()->contactFactory()->construct(this,
+                ReferencedHandles(connection(), HandleTypeContact, UIntList() << bareHandle),
+                features, attributes);
+        mPriv->contacts.insert(bareHandle, contact.data());
+
+        // do not call augment here as this is a fake contact
     }
 
     return contact;
