@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -58,6 +59,8 @@ public:
 
     Features supportedFeatures() const;
 
+    ContactListState state() const;
+
     Contacts allKnownContacts() const;
     QStringList allKnownGroups() const;
 
@@ -105,6 +108,8 @@ public:
             const Features &features = Features());
     PendingContacts *contactsForHandles(const ReferencedHandles &handles,
             const Features &features = Features());
+    PendingContacts *contactsForHandles(const HandleIdentifierMap &handles,
+            const Features &features = Features());
 
     PendingContacts *contactsForIdentifiers(const QStringList &identifiers,
             const Features &features = Features());
@@ -117,6 +122,8 @@ public:
     void requestContactAvatar(Contact *contact);
 
 Q_SIGNALS:
+    void stateChanged(Tp::ContactListState state);
+
     void presencePublicationRequested(const Tp::Contacts &contacts);
     // deprecated - carry redundant data which can be retrieved (meaningfully) from the Contacts
     // themselves (note: multiple contacts, but just a single message/details!)
@@ -142,14 +149,14 @@ protected:
     void connectNotify(const char *);
 
 private Q_SLOTS:
-    void onAliasesChanged(const Tp::AliasPairList &);
-    void doRequestAvatars();
-    void onAvatarUpdated(uint, const QString &);
-    void onAvatarRetrieved(uint, const QString &, const QByteArray &, const QString &);
-    void onPresencesChanged(const Tp::SimpleContactPresences &);
-    void onCapabilitiesChanged(const Tp::ContactCapabilitiesMap &);
-    void onLocationUpdated(uint, const QVariantMap &);
-    void onContactInfoChanged(uint, const Tp::ContactInfoFieldList &);
+    TELEPATHY_QT4_NO_EXPORT void onAliasesChanged(const Tp::AliasPairList &);
+    TELEPATHY_QT4_NO_EXPORT void doRequestAvatars();
+    TELEPATHY_QT4_NO_EXPORT void onAvatarUpdated(uint, const QString &);
+    TELEPATHY_QT4_NO_EXPORT void onAvatarRetrieved(uint, const QString &, const QByteArray &, const QString &);
+    TELEPATHY_QT4_NO_EXPORT void onPresencesChanged(const Tp::SimpleContactPresences &);
+    TELEPATHY_QT4_NO_EXPORT void onCapabilitiesChanged(const Tp::ContactCapabilitiesMap &);
+    TELEPATHY_QT4_NO_EXPORT void onLocationUpdated(uint, const QVariantMap &);
+    TELEPATHY_QT4_NO_EXPORT void onContactInfoChanged(uint, const Tp::ContactInfoFieldList &);
 
 private:
     class Roster;
@@ -157,18 +164,20 @@ private:
     friend class PendingContacts;
     friend class Roster;
 
-    ContactManager(Connection *parent);
+    TELEPATHY_QT4_NO_EXPORT ContactManager(Connection *parent);
 
-    ContactPtr ensureContact(const ReferencedHandles &handle,
+    TELEPATHY_QT4_NO_EXPORT ContactPtr ensureContact(const ReferencedHandles &handle,
             const Features &features,
             const QVariantMap &attributes);
+    TELEPATHY_QT4_NO_EXPORT ContactPtr ensureContact(uint bareHandle,
+            const QString &id, const Features &features);
 
-    void ensureTracking(const Feature &feature);
-    static QString featureToInterface(const Feature &feature);
+    TELEPATHY_QT4_NO_EXPORT static QString featureToInterface(const Feature &feature);
+    TELEPATHY_QT4_NO_EXPORT void ensureTracking(const Feature &feature);
 
-    PendingOperation *introspectRoster();
-    PendingOperation *introspectRosterGroups();
-    void resetRoster();
+    TELEPATHY_QT4_NO_EXPORT PendingOperation *introspectRoster();
+    TELEPATHY_QT4_NO_EXPORT PendingOperation *introspectRosterGroups();
+    TELEPATHY_QT4_NO_EXPORT void resetRoster();
 
     struct Private;
     friend struct Private;
