@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -175,7 +176,9 @@ Contact::Contact(ContactManager *manager, const ReferencedHandles &handle,
     : Object(),
       mPriv(new Private(this, manager, handle))
 {
-    augment(requestedFeatures, attributes);
+    mPriv->requestedFeatures.unite(requestedFeatures);
+    mPriv->id = qdbus_cast<QString>(attributes[
+            QLatin1String(TELEPATHY_INTERFACE_CONNECTION "/contact-id")]);
 }
 
 Contact::~Contact()
@@ -718,8 +721,10 @@ void Contact::setAvatarToken(const QString &token)
 
 void Contact::receiveAvatarData(const AvatarData &avatar)
 {
-    mPriv->avatarData = avatar;
-    emit avatarDataChanged(mPriv->avatarData);
+    if (mPriv->avatarData.fileName != avatar.fileName) {
+        mPriv->avatarData = avatar;
+        emit avatarDataChanged(mPriv->avatarData);
+    }
 }
 
 void Contact::receiveSimplePresence(const SimplePresence &presence)
