@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -60,9 +61,6 @@ public:
     FileTransferChannelCreationProperties &setLastModificationTime(
             const QDateTime &lastModificationTime);
 
-private:
-    friend class Account;
-
     /* mandatory parameters */
     QString suggestedFileName() const;
     QString contentType() const;
@@ -79,6 +77,7 @@ private:
     bool hasLastModificationTime() const;
     QDateTime lastModificationTime() const;
 
+private:
     struct Private;
     friend struct Private;
     QSharedDataPointer<Private> mPriv;
