@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2010 Nokia Corporation
+ * @copyright Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,7 +50,12 @@ public:
 
     virtual bool isValid() const { return false; }
 
-    virtual bool matches(const SharedPtr<T> &t) const { return false; }
+    virtual bool matches(const SharedPtr<T> &t) const
+    {
+        Q_UNUSED(t);
+
+        return false;
+    }
 
 protected:
     Filter() {}
