@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright © 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright © 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,8 +27,10 @@
 #include "TelepathyQt4/_gen/future-types.h"
 
 #include "TelepathyQt4/Channel"
+#include "TelepathyQt4/ChannelDispatcher"
 
 #include "TelepathyQt4/_gen/future-channel.h"
+#include "TelepathyQt4/_gen/future-channel-dispatcher.h"
 #include "TelepathyQt4/_gen/future-misc.h"
 
 #endif
