@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * @copyright Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -32,6 +33,7 @@
 #include <TelepathyQt4/ChannelDispatcher>
 #include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/ChannelRequest>
+#include <TelepathyQt4/ChannelRequestHints>
 #include <TelepathyQt4/ConnectionFactory>
 #include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/PendingFailure>
@@ -80,7 +82,7 @@ struct TELEPATHY_QT4_NO_EXPORT PendingChannelRequest::Private
  */
 PendingChannelRequest::PendingChannelRequest(const AccountPtr &account,
         const QVariantMap &requestedProperties, const QDateTime &userActionTime,
-        const QString &preferredHandler, bool create)
+        const QString &preferredHandler, bool create, const ChannelRequestHints &hints)
     : PendingOperation(account),
       mPriv(new Private(account->dbusConnection()))
 {
@@ -88,31 +90,60 @@ PendingChannelRequest::PendingChannelRequest(const AccountPtr &account,
         QString(QLatin1String("/%1")).arg(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER));
     channelDispatcherObjectPath.replace(QLatin1String("."), QLatin1String("/"));
     Client::ChannelDispatcherInterface *channelDispatcherInterface =
-        new Client::ChannelDispatcherInterface(mPriv->dbusConnection,
-                QLatin1String(TELEPATHY_INTERFACE_CHANNEL_DISPATCHER),
-                channelDispatcherObjectPath,
-                this);
-    QDBusPendingCallWatcher *watcher;
+        account->dispatcherInterface();
+
+    QDBusPendingCallWatcher *watcher = 0;
     if (create) {
-        watcher = new QDBusPendingCallWatcher(
-                channelDispatcherInterface->CreateChannel(
-                    QDBusObjectPath(account->objectPath()),
-                    requestedProperties,
-                    userActionTime.isNull() ? 0 : userActionTime.toTime_t(),
-                    preferredHandler), this);
-    }
-    else {
-        watcher = new QDBusPendingCallWatcher(
-                channelDispatcherInterface->EnsureChannel(
-                    QDBusObjectPath(account->objectPath()),
-                    requestedProperties,
-                    userActionTime.isNull() ? 0 : userActionTime.toTime_t(),
-                    preferredHandler), this);
+        if (hints.isValid()) {
+            if (account->supportsRequestHints()) {
+                watcher = new QDBusPendingCallWatcher(
+                    channelDispatcherInterface->CreateChannelWithHints(
+                        QDBusObjectPath(account->objectPath()),
+                        requestedProperties,
+                        userActionTime.isNull() ? 0 : userActionTime.toTime_t(),
+                        preferredHandler, hints.allHints()), this);
+            } else {
+                warning() << "Hints passed to channel request won't have an effect"
+                    << "because the Channel Dispatcher service in use is too old";
+            }
+        }
+
+        if (!watcher) {
+            watcher = new QDBusPendingCallWatcher(
+                    channelDispatcherInterface->CreateChannel(
+                        QDBusObjectPath(account->objectPath()),
+                        requestedProperties,
+                        userActionTime.isNull() ? 0 : userActionTime.toTime_t(),
+                        preferredHandler), this);
+        }
+    } else {
+        if (hints.isValid()) {
+            if (account->supportsRequestHints()) {
+                watcher = new QDBusPendingCallWatcher(
+                    channelDispatcherInterface->CreateChannelWithHints(
+                        QDBusObjectPath(account->objectPath()),
+                        requestedProperties,
+                        userActionTime.isNull() ? 0 : userActionTime.toTime_t(),
+                        preferredHandler, hints.allHints()), this);
+            } else {
+                warning() << "Hints passed to channel request won't have an effect"
+                    << "because the Channel Dispatcher service in use is too old";
+            }
+        }
+
+        if (!watcher) {
+            watcher = new QDBusPendingCallWatcher(
+                    channelDispatcherInterface->EnsureChannel(
+                        QDBusObjectPath(account->objectPath()),
+                        requestedProperties,
+                        userActionTime.isNull() ? 0 : userActionTime.toTime_t(),
+                        preferredHandler), this);
+        }
     }
 
-    // FIXME: This is a Qt bug fixed upstream, should be in the next Qt release.
-    //        We should not need to check watcher->isFinished() here, remove the
-    //        check when a fixed Qt version is released.
+    // TODO: This is a Qt bug fixed upstream, should be in the next Qt release.
+    //       We should not need to check watcher->isFinished() here, remove the
+    //       check when we depend on the fixed Qt version.
     if (watcher->isFinished()) {
         onWatcherFinished(watcher);
     } else {
@@ -191,7 +222,7 @@ void PendingChannelRequest::onWatcherFinished(QDBusPendingCallWatcher *watcher)
                     SIGNAL(failed(QString,QString)),
                     SLOT(setFinishedWithError(QString,QString)));
             connect(mPriv->channelRequest.data(),
-                    SIGNAL(succeeded()),
+                    SIGNAL(succeeded(Tp::ChannelPtr)),
                     SLOT(setFinished()));
 
             connect(mPriv->channelRequest->proceed(),
