@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2010 Nokia Corporation
+ * @copyright Copyright (C) 2008-2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,6 +39,7 @@ namespace Tp
 {
 
 class Connection;
+class HandledChannelNotifier;
 
 class TELEPATHY_QT4_EXPORT PendingChannel : public PendingOperation
 {
@@ -61,18 +63,33 @@ public:
 
     ChannelPtr channel() const;
 
+    HandledChannelNotifier *handledChannelNotifier() const;
+
 private Q_SLOTS:
-    void onCallCreateChannelFinished(QDBusPendingCallWatcher *watcher);
-    void onCallEnsureChannelFinished(QDBusPendingCallWatcher *watcher);
-    void onChannelReady(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onConnectionCreateChannelFinished(
+            QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onConnectionEnsureChannelFinished(
+            QDBusPendingCallWatcher *watcher);
+    TELEPATHY_QT4_NO_EXPORT void onChannelReady(Tp::PendingOperation *op);
+
+    TELEPATHY_QT4_NO_EXPORT void onHandlerError(const QString &errorName,
+            const QString &errorMessage);
+    TELEPATHY_QT4_NO_EXPORT void onHandlerChannelReceived(
+            const Tp::ChannelPtr &channel);
+    TELEPATHY_QT4_NO_EXPORT void onAccountCreateChannelFinished(
+            Tp::PendingOperation *op);
 
 private:
+    friend class Account;
     friend class ConnectionLowlevel;
 
-    PendingChannel(const ConnectionPtr &connection,
+    TELEPATHY_QT4_NO_EXPORT PendingChannel(const ConnectionPtr &connection,
             const QString &errorName, const QString &errorMessage);
-    PendingChannel(const ConnectionPtr &connection,
+    TELEPATHY_QT4_NO_EXPORT PendingChannel(const ConnectionPtr &connection,
             const QVariantMap &request, bool create);
+    TELEPATHY_QT4_NO_EXPORT PendingChannel(const AccountPtr &account,
+            const QVariantMap &request, const QDateTime &userActionTime,
+            bool create);
 
     struct Private;
     friend struct Private;
