@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * @copyright Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -80,6 +81,8 @@ struct TELEPATHY_QT4_NO_EXPORT PendingContacts::Private
     {
     }
 
+    void setFinished();
+
     // Public object
     PendingContacts *parent;
 
@@ -104,6 +107,21 @@ struct TELEPATHY_QT4_NO_EXPORT PendingContacts::Private
     ReferencedHandles handlesToInspect;
 };
 
+void PendingContacts::Private::setFinished()
+{
+    ConnectionLowlevelPtr connLowlevel = manager->connection()->lowlevel();
+    UIntList handles = invalidHandles;
+    foreach (uint handle, handles) {
+        if (connLowlevel->hasContactId(handle)) {
+            satisfyingContacts.insert(handle, manager->ensureContact(handle,
+                        connLowlevel->contactId(handle), features));
+            invalidHandles.removeOne(handle);
+        }
+    }
+
+    parent->setFinished();
+}
+
 PendingContacts::PendingContacts(const ContactManagerPtr &manager,
         const UIntList &handles, const Features &features,
         const QStringList &interfaces,
@@ -389,7 +407,7 @@ void PendingContacts::onNestedFinished(PendingOperation *operation)
 
     mPriv->contacts = mPriv->nested->contacts();
     mPriv->nested = 0;
-    setFinished();
+    mPriv->setFinished();
 }
 
 void PendingContacts::onInspectHandlesFinished(QDBusPendingCallWatcher *watcher)
@@ -429,7 +447,7 @@ void PendingContacts::allAttributesFetched()
         }
     }
 
-    setFinished();
+    mPriv->setFinished();
 }
 
 } // Tp
