@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2009 Nokia Corporation
+ * @copyright Copyright (C) 2008-2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -71,7 +72,7 @@ protected Q_SLOTS:
     void setFinishedWithError(const QDBusError &error);
 
 private Q_SLOTS:
-    void emitFinished();
+    TELEPATHY_QT4_NO_EXPORT void emitFinished();
 
 private:
     friend class ContactManager;
