@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2010 Nokia Corporation
+ * @copyright Copyright (C) 2010 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2010 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -47,9 +48,8 @@ struct TELEPATHY_QT4_NO_EXPORT Profile::Private
     void setFileName(const QString &fileName);
 
     void lookupProfile();
-    bool parse(const QString &fileName);
-
-    void setError(const QString &errorMessage);
+    bool parse(QFile *file);
+    void invalidate();
 
     struct Data
     {
@@ -422,6 +422,8 @@ Profile::Private::Private()
 
 void Profile::Private::setServiceName(const QString &serviceName_)
 {
+    invalidate();
+
     allowNonIMType = false;
     serviceName = serviceName_;
     lookupProfile();
@@ -429,57 +431,81 @@ void Profile::Private::setServiceName(const QString &serviceName_)
 
 void Profile::Private::setFileName(const QString &fileName)
 {
+    invalidate();
+
     allowNonIMType = true;
     QFileInfo fi(fileName);
     serviceName = fi.baseName();
-    parse(fileName);
+
+    debug() << "Loading profile file" << fileName;
+
+    QFile file(fileName);
+    if (!file.exists()) {
+        warning() << QString(QLatin1String("Error parsing profile file %1: file does not exist"))
+            .arg(file.fileName());
+        return;
+    }
+
+    if (!file.open(QFile::ReadOnly)) {
+        warning() << QString(QLatin1String("Error parsing profile file %1: "
+                    "cannot open file for readonly access"))
+            .arg(file.fileName());
+        return;
+    }
+
+    if (parse(&file)) {
+        debug() << "Profile file" << fileName << "loaded successfully";
+    }
 }
 
 void Profile::Private::lookupProfile()
 {
-    QStringList searchDirs = Profile::searchDirs();
+    debug() << "Searching profile for service" << serviceName;
 
+    QStringList searchDirs = Profile::searchDirs();
+    bool found = false;
     foreach (const QString searchDir, searchDirs) {
         QString fileName = searchDir + serviceName + QLatin1String(".profile");
-        if (parse(fileName)) {
-            return;
+
+        QFile file(fileName);
+        if (!file.exists()) {
+            continue;
         }
+
+        if (!file.open(QFile::ReadOnly)) {
+            continue;
+        }
+
+        if (parse(&file)) {
+            debug() << "Profile for service" << serviceName << "found:" << fileName;
+            found = true;
+            break;
+        }
+    }
+
+    if (!found) {
+        debug() << "Cannot find valid profile for service" << serviceName;
     }
 }
 
-bool Profile::Private::parse(const QString &fileName)
+bool Profile::Private::parse(QFile *file)
 {
-    data.clear();
-    valid = false;
+    invalidate();
+
     fake = false;
-
-    QFile file(fileName);
-    if (!file.exists()) {
-        setError(QString(QLatin1String("Error parsing profile file %1: "
-                        "file does not exist"))
-                .arg(file.fileName()));
-        return false;
-    }
-
-    if (!file.open(QFile::ReadOnly)) {
-        setError(QString(QLatin1String("Error parsing profile file %1: "
-                        "cannot open file for readonly access"))
-                .arg(file.fileName()));
-        return false;
-    }
-
-    QFileInfo fi(file.fileName());
+    QFileInfo fi(file->fileName());
     XmlHandler xmlHandler(serviceName, allowNonIMType, &data);
 
     QXmlSimpleReader xmlReader;
     xmlReader.setContentHandler(&xmlHandler);
     xmlReader.setErrorHandler(&xmlHandler);
 
-    QXmlInputSource xmlInputSource(&file);
+    QXmlInputSource xmlInputSource(file);
     if (!xmlReader.parse(xmlInputSource)) {
-        setError(QString(QLatin1String("Error parsing profile file %1: %2"))
-                .arg(file.fileName())
-                .arg(xmlHandler.errorString()));
+        warning() << QString(QLatin1String("Error parsing profile file %1: %2"))
+            .arg(file->fileName())
+            .arg(xmlHandler.errorString());
+        invalidate();
         return false;
     }
 
@@ -487,9 +513,8 @@ bool Profile::Private::parse(const QString &fileName)
     return true;
 }
 
-void Profile::Private::setError(const QString &errorMessage)
+void Profile::Private::invalidate()
 {
-    warning() << errorMessage;
     valid = false;
     data.clear();
 }
@@ -1132,9 +1157,23 @@ void Profile::Presence::setIconName(const QString &iconName)
 }
 
 /**
- * Return the default text delivered with this presence.
+ * Return whether user-defined text-message can be attached to this presence.
  *
- * \return The default text delivered with this presence.
+ * \return \c true if user-defined text-message can be attached to this presence, \c false
+ *         otherwise.
+ */
+bool Profile::Presence::canHaveStatusMessage() const
+{
+    if (mPriv->message == QLatin1String("1") ||
+        mPriv->message == QLatin1String("true")) {
+        return true;
+    }
+
+    return false;
+}
+
+/**
+ * \deprecated Use canHaveStatusMessage() instead.
  */
 QString Profile::Presence::message() const
 {
