@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -267,12 +268,6 @@ void ReadinessHelper::Private::iterateIntrospection()
         return;
     }
 
-    if (!supportedStatuses.contains(currentStatus)) {
-        debug() << "ignoring iterate introspection for status" << currentStatus;
-        // don't do anything just now to avoid spurious becomeReady finishes
-        return;
-    }
-
     // take care to flag anything with dependencies in missing, and the
     // stuff depending on them, as missing
     for (Introspectables::const_iterator i = introspectables.constBegin();
