@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008-2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008-2009 Nokia Corporation
+ * @copyright Copyright (C) 2008-2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008-2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -246,7 +247,7 @@ private:
     friend class PendingContacts;
     friend class PendingHandles;
 
-    ReferencedHandles(const ConnectionPtr &connection,
+    TELEPATHY_QT4_NO_EXPORT ReferencedHandles(const ConnectionPtr &connection,
             HandleType handleType, const UIntList& handles);
 
     struct Private;
