@@ -1,7 +1,9 @@
-/* StreamedMedia channel client-side proxy
+/**
+ * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -51,15 +53,15 @@ public:
     StreamedMediaStreams streams() const;
 
 private Q_SLOTS:
-    void gotStreams(QDBusPendingCallWatcher *op);
+    TELEPATHY_QT4_NO_EXPORT void gotStreams(QDBusPendingCallWatcher *op);
 
-    void onStreamRemoved(const Tp::StreamedMediaStreamPtr &stream);
-    void onStreamReady(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onStreamRemoved(const Tp::StreamedMediaStreamPtr &stream);
+    TELEPATHY_QT4_NO_EXPORT void onStreamReady(Tp::PendingOperation *op);
 
 private:
     friend class StreamedMediaChannel;
 
-    PendingStreamedMediaStreams(const StreamedMediaChannelPtr &channel,
+    TELEPATHY_QT4_NO_EXPORT PendingStreamedMediaStreams(const StreamedMediaChannelPtr &channel,
             const ContactPtr &contact,
             const QList<MediaStreamType> &types);
 
@@ -117,7 +119,7 @@ Q_SIGNALS:
     void remoteSendingStateChanged(Tp::StreamedMediaStream::SendingState remoteSendingState);
 
 private Q_SLOTS:
-    void gotContact(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void gotContact(Tp::PendingOperation *op);
 
 private:
     friend class PendingStreamedMediaStreams;
@@ -125,10 +127,10 @@ private:
 
     static const Feature FeatureCore;
 
-    StreamedMediaStream(const StreamedMediaChannelPtr &channel, const MediaStreamInfo &info);
+    TELEPATHY_QT4_NO_EXPORT StreamedMediaStream(const StreamedMediaChannelPtr &channel, const MediaStreamInfo &info);
 
-    void gotDirection(uint direction, uint pendingSend);
-    void gotStreamState(uint state);
+    TELEPATHY_QT4_NO_EXPORT void gotDirection(uint direction, uint pendingSend);
+    TELEPATHY_QT4_NO_EXPORT void gotStreamState(uint state);
 
     struct Private;
     friend struct Private;
@@ -198,17 +200,17 @@ protected:
             const Feature &coreFeature = StreamedMediaChannel::FeatureCore);
 
 private Q_SLOTS:
-    void onStreamReady(Tp::PendingOperation *op);
+    TELEPATHY_QT4_NO_EXPORT void onStreamReady(Tp::PendingOperation *op);
 
-    void gotStreams(QDBusPendingCallWatcher *);
-    void onStreamAdded(uint, uint, uint);
-    void onStreamRemoved(uint);
-    void onStreamDirectionChanged(uint, uint, uint);
-    void onStreamStateChanged(uint streamId, uint streamState);
-    void onStreamError(uint, uint, const QString &);
+    TELEPATHY_QT4_NO_EXPORT void gotStreams(QDBusPendingCallWatcher *);
+    TELEPATHY_QT4_NO_EXPORT void onStreamAdded(uint, uint, uint);
+    TELEPATHY_QT4_NO_EXPORT void onStreamRemoved(uint);
+    TELEPATHY_QT4_NO_EXPORT void onStreamDirectionChanged(uint, uint, uint);
+    TELEPATHY_QT4_NO_EXPORT void onStreamStateChanged(uint streamId, uint streamState);
+    TELEPATHY_QT4_NO_EXPORT void onStreamError(uint, uint, const QString &);
 
-    void gotLocalHoldState(QDBusPendingCallWatcher *);
-    void onLocalHoldStateChanged(uint, uint);
+    TELEPATHY_QT4_NO_EXPORT void gotLocalHoldState(QDBusPendingCallWatcher *);
+    TELEPATHY_QT4_NO_EXPORT void onLocalHoldStateChanged(uint, uint);
 
 private:
     friend class PendingStreamedMediaStreams;
