@@ -1,7 +1,9 @@
-/* Text channel client-side proxy
+/**
+ * This file is part of TelepathyQt4
  *
- * Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2009 Nokia Corporation
+ * @copyright Copyright (C) 2009 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2009 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -20,11 +22,12 @@
 
 #include <TelepathyQt4/TextChannel>
 
-#include <QDateTime>
-
 #include "TelepathyQt4/_gen/text-channel.moc.hpp"
 
+#include "TelepathyQt4/debug-internal.h"
+
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/Message>
 #include <TelepathyQt4/PendingContacts>
@@ -33,74 +36,11 @@
 #include <TelepathyQt4/ReceivedMessage>
 #include <TelepathyQt4/ReferencedHandles>
 
-#include "TelepathyQt4/debug-internal.h"
+#include <QDateTime>
 
 namespace Tp
 {
 
-struct TELEPATHY_QT4_NO_EXPORT PendingSendMessage::Private
-{
-    inline Private(const Message &message);
-    QString token;
-    Message message;
-};
-
-inline PendingSendMessage::Private::Private(const Message &message)
-    : token(QLatin1String("")), message(message)
-{
-}
-
-PendingSendMessage::PendingSendMessage(const TextChannelPtr &channel, const Message &message)
-    : PendingOperation(channel),
-      mPriv(new Private(message))
-{
-}
-
-PendingSendMessage::~PendingSendMessage()
-{
-    delete mPriv;
-}
-
-TextChannelPtr PendingSendMessage::channel() const
-{
-    return TextChannelPtr(qobject_cast<TextChannel*>((TextChannel*) object().data()));
-}
-
-QString PendingSendMessage::sentMessageToken() const
-{
-    return mPriv->token;
-}
-
-Message PendingSendMessage::message() const
-{
-    return mPriv->message;
-}
-
-void PendingSendMessage::onTextSent(QDBusPendingCallWatcher *watcher)
-{
-    QDBusPendingReply<> reply = *watcher;
-
-    if (reply.isError()) {
-        setFinishedWithError(reply.error());
-    } else {
-        setFinished();
-    }
-    watcher->deleteLater();
-}
-
-void PendingSendMessage::onMessageSent(QDBusPendingCallWatcher *watcher)
-{
-    QDBusPendingReply<QString> reply = *watcher;
-
-    if (reply.isError()) {
-        setFinishedWithError(reply.error());
-    } else {
-        mPriv->token = reply.value();
-        setFinished();
-    }
-    watcher->deleteLater();
-}
-
 struct TELEPATHY_QT4_NO_EXPORT TextChannel::Private
 {
     Private(TextChannel *parent);
@@ -396,7 +336,7 @@ void TextChannel::Private::processMessageQueue()
     // to incoming messages
     while (!incompleteMessages.isEmpty()) {
         const MessageEvent *e = incompleteMessages.first();
-        debug() << "MessageEvent:" << e;
+        debug() << "MessageEvent:" << reinterpret_cast<const void *>(e);
 
         if (e->isMessage) {
             if (e->message.senderHandle() != 0 &&
@@ -441,13 +381,13 @@ void TextChannel::Private::processMessageQueue()
 
     // What Contact objects do we need in order to proceed, ignoring those
     // for which we've already sent a request?
-    QSet<uint> contactsRequired;
+    HandleIdentifierMap contactsRequired;
     foreach (const MessageEvent *e, incompleteMessages) {
         if (e->isMessage) {
             uint handle = e->message.senderHandle();
             if (handle != 0 && !e->message.sender()
                     && !awaitingContacts.contains(handle)) {
-                contactsRequired << handle;
+                contactsRequired.insert(handle, e->message.senderId());
             }
         }
     }
@@ -456,19 +396,22 @@ void TextChannel::Private::processMessageQueue()
         return;
     }
 
-    parent->connect(parent->connection()->contactManager()->contactsForHandles(
-                contactsRequired.toList()),
+    ConnectionPtr conn = parent->connection();
+    conn->lowlevel()->injectContactIds(contactsRequired);
+
+    parent->connect(conn->contactManager()->contactsForHandles(
+                contactsRequired.keys()),
             SIGNAL(finished(Tp::PendingOperation*)),
             SLOT(onContactsFinished(Tp::PendingOperation*)));
 
-    awaitingContacts |= contactsRequired;
+    awaitingContacts |= contactsRequired.keys().toSet();
 }
 
 void TextChannel::Private::processChatStateQueue()
 {
     while (!chatStateQueue.isEmpty()) {
         const ChatStateEvent *e = chatStateQueue.first();
-        debug() << "ChatStateEvent:" << e;
+        debug() << "ChatStateEvent:" << reinterpret_cast<const void *>(e);
 
         if (e->contact.isNull()) {
             // the chat state Contact object wasn't retrieved yet, but needs
@@ -500,6 +443,8 @@ void TextChannel::Private::processChatStateQueue()
         return;
     }
 
+    // TODO: pass id hints to ContactManager if we ever gain support to retrieve contact ids
+    //       from ChatState.
     parent->connect(parent->connection()->contactManager()->contactsForHandles(
                 contactsRequired.toList()),
             SIGNAL(finished(Tp::PendingOperation*)),
