@@ -1,8 +1,9 @@
-/*
+/**
  * This file is part of TelepathyQt4
  *
- * Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
- * Copyright (C) 2008 Nokia Corporation
+ * @copyright Copyright (C) 2008 Collabora Ltd. <http://www.collabora.co.uk/>
+ * @copyright Copyright (C) 2008 Nokia Corporation
+ * @license LGPL 2.1
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -65,17 +66,23 @@ class ConnectionManagerLowlevel;
 class Contact;
 class ContactFactory;
 class ContactManager;
+class ContactMessenger;
 class ContactSearchChannel;
 class DBusProxy;
 class FileTransferChannel;
 class IncomingFileTransferChannel;
+class IncomingStreamTubeChannel;
 class OutgoingFileTransferChannel;
+class OutgoingStreamTubeChannel;
 class Profile;
 class ProfileManager;
 class RoomListChannel;
+class SimpleTextObserver;
 class StreamedMediaChannel;
 class StreamedMediaStream;
+class StreamTubeChannel;
 class TextChannel;
+class TubeChannel;
 
 typedef SharedPtr<AbstractClient> AbstractClientPtr;
 typedef SharedPtr<Account> AccountPtr;
@@ -109,17 +116,23 @@ typedef QSet<ContactPtr> Contacts;
 typedef SharedPtr<ContactFactory> ContactFactoryPtr;
 typedef SharedPtr<const ContactFactory> ContactFactoryConstPtr;
 typedef SharedPtr<ContactManager> ContactManagerPtr;
+typedef SharedPtr<ContactMessenger> ContactMessengerPtr;
 typedef SharedPtr<ContactSearchChannel> ContactSearchChannelPtr;
 typedef SharedPtr<DBusProxy> DBusProxyPtr;
 typedef SharedPtr<FileTransferChannel> FileTransferChannelPtr;
 typedef SharedPtr<IncomingFileTransferChannel> IncomingFileTransferChannelPtr;
+typedef SharedPtr<IncomingStreamTubeChannel> IncomingStreamTubeChannelPtr;
 typedef SharedPtr<OutgoingFileTransferChannel> OutgoingFileTransferChannelPtr;
+typedef SharedPtr<OutgoingStreamTubeChannel> OutgoingStreamTubeChannelPtr;
 typedef SharedPtr<Profile> ProfilePtr;
 typedef SharedPtr<ProfileManager> ProfileManagerPtr;
 typedef SharedPtr<RoomListChannel> RoomListChannelPtr;
+typedef SharedPtr<SimpleTextObserver> SimpleTextObserverPtr;
 typedef SharedPtr<StreamedMediaChannel> StreamedMediaChannelPtr;
 typedef SharedPtr<StreamedMediaStream> StreamedMediaStreamPtr;
+typedef SharedPtr<StreamTubeChannel> StreamTubeChannelPtr;
 typedef SharedPtr<TextChannel> TextChannelPtr;
+typedef SharedPtr<TubeChannel> TubeChannelPtr;
 
 template<typename T1 = MethodInvocationContextTypes::Nil, typename T2 = MethodInvocationContextTypes::Nil,
          typename T3 = MethodInvocationContextTypes::Nil, typename T4 = MethodInvocationContextTypes::Nil,
