@@ -5,8 +5,13 @@
 
 #include <QtTest/QtTest>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
 #include <TelepathyQt4/Channel>
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/PendingChannel>
 #include <TelepathyQt4/PendingHandles>
 #include <TelepathyQt4/PendingReady>
@@ -30,8 +35,8 @@ public:
     { }
 
 protected Q_SLOTS:
-    void expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason);
-    void expectConnInvalidated();
+    void expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason);
+    void expectInvalidated(const QString &reason);
     void expectPendingHandleFinished(Tp::PendingOperation*);
     void expectCreateChannelFinished(Tp::PendingOperation *);
     void expectEnsureChannelFinished(Tp::PendingOperation *);
@@ -56,19 +61,19 @@ private:
     uint mHandle;
 };
 
-void TestChanBasics::expectConnReady(Tp::Connection::Status newStatus,
+void TestChanBasics::expectConnReady(Tp::ConnectionStatus newStatus,
         Tp::ConnectionStatusReason newStatusReason)
 {
     qDebug() << "connection changed to status" << newStatus;
     switch (newStatus) {
-    case Connection::StatusDisconnected:
+    case ConnectionStatusDisconnected:
         qWarning() << "Disconnected";
         mLoop->exit(1);
         break;
-    case Connection::StatusConnecting:
+    case ConnectionStatusConnecting:
         /* do nothing */
         break;
-    case Connection::StatusConnected:
+    case ConnectionStatusConnected:
         qDebug() << "Ready";
         mLoop->exit(0);
         break;
@@ -80,8 +85,9 @@ void TestChanBasics::expectConnReady(Tp::Connection::Status newStatus,
     }
 }
 
-void TestChanBasics::expectConnInvalidated()
+void TestChanBasics::expectInvalidated(const QString &reason)
 {
+    qDebug() << sender() << "invalidated:" << reason;
     mLoop->exit(0);
 }
 
@@ -199,10 +205,12 @@ void TestChanBasics::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    mConn->requestConnect();
+    mConn->lowlevel()->requestConnect();
 
     Features features = Features() << Connection::FeatureSelfContact;
     QVERIFY(connect(mConn->becomeReady(features),
@@ -211,24 +219,24 @@ void TestChanBasics::initTestCase()
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(features), true);
 
-    if (mConn->status() != Connection::StatusConnected) {
+    if (mConn->status() != ConnectionStatusConnected) {
         QVERIFY(connect(mConn.data(),
-                        SIGNAL(statusChanged(Tp::Connection::Status, Tp::ConnectionStatusReason)),
-                        SLOT(expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason))));
+                        SIGNAL(statusChanged(Tp::ConnectionStatus, Tp::ConnectionStatusReason)),
+                        SLOT(expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason))));
         QCOMPARE(mLoop->exec(), 0);
         QVERIFY(disconnect(mConn.data(),
-                           SIGNAL(statusChanged(Tp::Connection::Status, Tp::ConnectionStatusReason)),
+                           SIGNAL(statusChanged(Tp::ConnectionStatus, Tp::ConnectionStatusReason)),
                            this,
-                           SLOT(expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason))));
-        QCOMPARE(mConn->status(), Connection::StatusConnected);
+                           SLOT(expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason))));
+        QCOMPARE(mConn->status(), ConnectionStatusConnected);
     }
-
-    QVERIFY(mConn->requestsInterface() != 0);
 }
 
 void TestChanBasics::init()
 {
     initImpl();
+
+    mChan.reset();
 }
 
 void TestChanBasics::testRequestHandle()
@@ -237,7 +245,7 @@ void TestChanBasics::testRequestHandle()
     QStringList ids = QStringList() << QLatin1String("alice");
 
     // Request handles for the identifiers and wait for the request to process
-    PendingHandles *pending = mConn->requestHandles(Tp::HandleTypeContact, ids);
+    PendingHandles *pending = mConn->lowlevel()->requestHandles(Tp::HandleTypeContact, ids);
     QVERIFY(connect(pending,
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectPendingHandleFinished(Tp::PendingOperation*))));
@@ -258,36 +266,34 @@ void TestChanBasics::testCreateChannel()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    mHandle);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    if (mChan) {
-        QVERIFY(connect(mChan->becomeReady(),
-                        SIGNAL(finished(Tp::PendingOperation*)),
-                        SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-        QCOMPARE(mLoop->exec(), 0);
-        QCOMPARE(mChan->isReady(), true);
-        QCOMPARE(mChan->isRequested(), true);
-        QCOMPARE(mChan->groupCanAddContacts(), false);
-        QCOMPARE(mChan->groupCanRemoveContacts(), false);
-        QCOMPARE(mChan->initiatorContact()->id(), QString(QLatin1String("me@example.com")));
-        QCOMPARE(mChan->groupSelfContact()->id(), QString(QLatin1String("me@example.com")));
-        QCOMPARE(mChan->groupSelfContact(), mConn->selfContact());
+    Q_ASSERT(!mChan.isNull());
 
-        QStringList ids;
-        Q_FOREACH (const ContactPtr &contact, mChan->groupContacts()) {
-            ids << contact->id();
-        }
-        ids.sort();
-        QStringList toCheck = QStringList() << QLatin1String("me@example.com")
-            << QLatin1String("alice");
-        toCheck.sort();
-        QCOMPARE(ids, toCheck);
+    QVERIFY(connect(mChan->becomeReady(),
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(mChan->isReady(), true);
+    QCOMPARE(mChan->isRequested(), true);
+    QCOMPARE(mChan->groupCanAddContacts(), false);
+    QCOMPARE(mChan->groupCanRemoveContacts(), false);
+    QCOMPARE(mChan->initiatorContact()->id(), QString(QLatin1String("me@example.com")));
+    QCOMPARE(mChan->groupSelfContact()->id(), QString(QLatin1String("me@example.com")));
+    QCOMPARE(mChan->groupSelfContact(), mConn->selfContact());
 
-        mChan.reset();
+    QStringList ids;
+    Q_FOREACH (const ContactPtr &contact, mChan->groupContacts()) {
+        ids << contact->id();
     }
+    ids.sort();
+    QStringList toCheck = QStringList() << QLatin1String("me@example.com")
+        << QLatin1String("alice");
+    toCheck.sort();
+    QCOMPARE(ids, toCheck);
 }
 
 void TestChanBasics::testEnsureChannel()
@@ -299,42 +305,40 @@ void TestChanBasics::testEnsureChannel()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    mHandle);
-    QVERIFY(connect(mConn->ensureChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->ensureChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectEnsureChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
 
-    if (mChan) {
-        QVERIFY(connect(mChan->becomeReady(),
-                        SIGNAL(finished(Tp::PendingOperation*)),
-                        SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-        QCOMPARE(mLoop->exec(), 0);
-        QCOMPARE(mChan->isReady(), true);
-        QCOMPARE(mChan->isRequested(), true);
-        QCOMPARE(mChan->groupCanAddContacts(), false);
-        QCOMPARE(mChan->groupCanRemoveContacts(), false);
-        QCOMPARE(mChan->initiatorContact()->id(), QString(QLatin1String("me@example.com")));
-        QCOMPARE(mChan->groupSelfContact()->id(), QString(QLatin1String("me@example.com")));
-        QCOMPARE(mChan->groupSelfContact(), mConn->selfContact());
+    QVERIFY(!mChan.isNull());
 
-        QStringList ids;
-        Q_FOREACH (const ContactPtr &contact, mChan->groupContacts()) {
-            ids << contact->id();
-        }
-        ids.sort();
-        QStringList toCheck = QStringList() << QLatin1String("me@example.com")
-            << QLatin1String("alice");
-        toCheck.sort();
-        QCOMPARE(ids, toCheck);
+    QVERIFY(connect(mChan->becomeReady(),
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(mChan->isReady(), true);
+    QCOMPARE(mChan->isRequested(), true);
+    QCOMPARE(mChan->groupCanAddContacts(), false);
+    QCOMPARE(mChan->groupCanRemoveContacts(), false);
+    QCOMPARE(mChan->initiatorContact()->id(), QString(QLatin1String("me@example.com")));
+    QCOMPARE(mChan->groupSelfContact()->id(), QString(QLatin1String("me@example.com")));
+    QCOMPARE(mChan->groupSelfContact(), mConn->selfContact());
 
-        QVERIFY(connect(mChan->requestClose(),
-                        SIGNAL(finished(Tp::PendingOperation*)),
-                        SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
-        QCOMPARE(mLoop->exec(), 0);
-        QCOMPARE(mChan->isValid(), false);
-
-        mChan.reset();
+    QStringList ids;
+    Q_FOREACH (const ContactPtr &contact, mChan->groupContacts()) {
+        ids << contact->id();
     }
+    ids.sort();
+    QStringList toCheck = QStringList() << QLatin1String("me@example.com")
+        << QLatin1String("alice");
+    toCheck.sort();
+    QCOMPARE(ids, toCheck);
+
+    QVERIFY(connect(mChan->requestClose(),
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+    QCOMPARE(mChan->isValid(), false);
 }
 
 void TestChanBasics::cleanup()
@@ -346,7 +350,7 @@ void TestChanBasics::cleanupTestCase()
 {
     if (mConn) {
         // Disconnect and wait for the readiness change
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
@@ -355,14 +359,22 @@ void TestChanBasics::cleanupTestCase()
             QVERIFY(connect(mConn.data(),
                             SIGNAL(invalidated(Tp::DBusProxy *,
                                                const QString &, const QString &)),
-                            SLOT(expectConnInvalidated())));
+                            SLOT(expectInvalidated(QString))));
             QCOMPARE(mLoop->exec(), 0);
         }
     }
 
-    if (mConnService != 0) {
-        g_object_unref(mConnService);
-        mConnService = 0;
+    if (mChan) {
+        if (mChan->isValid()) {
+            QVERIFY(connect(mChan.data(),
+                        SIGNAL(invalidated(Tp::DBusProxy *,
+                                const QString &, const QString &)),
+                        SLOT(expectInvalidated(QString))));
+            QCOMPARE(mLoop->exec(), 0);
+        }
+
+        QVERIFY(mChan->invalidationReason() == TP_QT4_ERROR_CANCELLED ||
+                mChan->invalidationReason() == TP_QT4_ERROR_ORPHANED);
     }
 
     cleanupTestCaseImpl();
