@@ -7,12 +7,16 @@
 #include <QString>
 #include <QVariantMap>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
 #include <TelepathyQt4/Account>
 #include <TelepathyQt4/AccountFactory>
 #include <TelepathyQt4/AccountManager>
 #include <TelepathyQt4/AbstractClientHandler>
 #include <TelepathyQt4/AbstractClientObserver>
 #include <TelepathyQt4/Channel>
+#include <TelepathyQt4/ChannelClassSpec>
+#include <TelepathyQt4/ChannelClassSpecList>
 #include <TelepathyQt4/ChannelDispatchOperation>
 #include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/ChannelRequest>
@@ -22,11 +26,13 @@
 #include <TelepathyQt4/ClientRegistrar>
 #include <TelepathyQt4/Connection>
 #include <TelepathyQt4/ConnectionFactory>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/Debug>
 #include <TelepathyQt4/MethodInvocationContext>
 #include <TelepathyQt4/PendingAccount>
 #include <TelepathyQt4/PendingReady>
+#include <TelepathyQt4/TextChannel>
 #include <TelepathyQt4/Types>
 
 #include <telepathy-glib/debug.h>
@@ -208,8 +214,8 @@ class MyClient : public QObject,
     Q_OBJECT
 
 public:
-    static AbstractClientPtr create(const ChannelClassList &channelFilter,
-            const QStringList &capabilities,
+    static AbstractClientPtr create(const ChannelClassSpecList &channelFilter,
+            const AbstractClientHandler::Capabilities &capabilities,
             bool bypassApproval = false,
             bool wantsRequestNotification = false)
     {
@@ -218,8 +224,8 @@ public:
                         bypassApproval, wantsRequestNotification)));
     }
 
-    MyClient(const ChannelClassList &channelFilter,
-             const QStringList &capabilities,
+    MyClient(const ChannelClassSpecList &channelFilter,
+             const AbstractClientHandler::Capabilities &capabilities,
              bool bypassApproval = false,
              bool wantsRequestNotification = false)
         : AbstractClientObserver(channelFilter),
@@ -239,7 +245,7 @@ public:
             const QList<ChannelPtr> &channels,
             const ChannelDispatchOperationPtr &dispatchOperation,
             const QList<ChannelRequestPtr> &requestsSatisfied,
-            const QVariantMap &observerInfo)
+            const AbstractClientObserver::ObserverInfo &observerInfo)
     {
         mObserveChannelsAccount = account;
         mObserveChannelsConnection = connection;
@@ -253,10 +259,9 @@ public:
     }
 
     void addDispatchOperation(const MethodInvocationContextPtr<> &context,
-            const QList<ChannelPtr> &channels,
             const ChannelDispatchOperationPtr &dispatchOperation)
     {
-        mAddDispatchOperationChannels = channels;
+        mAddDispatchOperationChannels = dispatchOperation->channels();
         mAddDispatchOperationDispatchOperation = dispatchOperation;
 
         context->setFinished();
@@ -274,7 +279,7 @@ public:
             const QList<ChannelPtr> &channels,
             const QList<ChannelRequestPtr> &requestsSatisfied,
             const QDateTime &userActionTime,
-            const QVariantMap &handlerInfo)
+            const AbstractClientHandler::HandlerInfo &handlerInfo)
     {
         mHandleChannelsAccount = account;
         mHandleChannelsConnection = connection;
@@ -314,7 +319,7 @@ public:
     QList<ChannelPtr> mObserveChannelsChannels;
     ChannelDispatchOperationPtr mObserveChannelsDispatchOperation;
     QList<ChannelRequestPtr> mObserveChannelsRequestsSatisfied;
-    QVariantMap mObserveChannelsObserverInfo;
+    AbstractClientObserver::ObserverInfo mObserveChannelsObserverInfo;
 
     QList<ChannelPtr> mAddDispatchOperationChannels;
     ChannelDispatchOperationPtr mAddDispatchOperationDispatchOperation;
@@ -325,7 +330,7 @@ public:
     QList<ChannelPtr> mHandleChannelsChannels;
     QList<ChannelRequestPtr> mHandleChannelsRequestsSatisfied;
     QDateTime mHandleChannelsUserActionTime;
-    QVariantMap mHandleChannelsHandlerInfo;
+    AbstractClientHandler::HandlerInfo mHandleChannelsHandlerInfo;
     ChannelRequestPtr mAddRequestRequest;
     ChannelRequestPtr mRemoveRequestRequest;
     QString mRemoveRequestErrorName;
@@ -393,7 +398,7 @@ private:
     QString mChannelRequestPath;
     ChannelDispatchOperationAdaptor *mCDO;
     QString mCDOPath;
-    QStringList mClientCapabilities;
+    AbstractClientHandler::Capabilities mClientCapabilities;
     AbstractClientPtr mClientObject1;
     QString mClientObject1BusName;
     QString mClientObject1Path;
@@ -419,9 +424,23 @@ void TestClientFactories::initTestCase()
 
     QDBusConnection bus = QDBusConnection::sessionBus();
 
+    ChannelFactoryPtr chanFact = ChannelFactory::create(bus);
+
+    chanFact->addFeaturesForTextChats(TextChannel::FeatureChatState |
+            TextChannel::FeatureMessageQueue);
+    chanFact->addCommonFeatures(Channel::FeatureCore);
+
+    QCOMPARE(chanFact->commonFeatures().size(), 1);
+
+    QCOMPARE(chanFact->featuresForTextChats().size(), 3);
+    QVERIFY(chanFact->featuresForTextChats().contains(TextChannel::FeatureMessageQueue));
+    QVERIFY(chanFact->featuresForTextChats().contains(Channel::FeatureCore));
+    QVERIFY(!chanFact->featuresForTextChats().contains(TextChannel::FeatureMessageSentSignal));
+
     mAM = AccountManager::create(AccountFactory::create(bus, Account::FeatureCore),
             ConnectionFactory::create(bus,
-                Connection::FeatureCore | Connection::FeatureSimplePresence));
+                Connection::FeatureCore | Connection::FeatureSimplePresence),
+            chanFact);
     PendingReady *amReadyOp = mAM->becomeReady();
     QVERIFY(amReadyOp != NULL);
     QVERIFY(connect(amReadyOp,
@@ -467,11 +486,11 @@ void TestClientFactories::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = ConnectionPtr::dynamicCast(mAccount->connectionFactory()->proxy(mConnName, mConnPath,
+    mConn = ConnectionPtr::qObjectCast(mAccount->connectionFactory()->proxy(mConnName, mConnPath,
                 ChannelFactory::create(bus), ContactFactory::create())->proxy());
     QCOMPARE(mConn->isReady(), false);
 
-    PendingReady *mConnReady = mConn->requestConnect();
+    PendingReady *mConnReady = mConn->lowlevel()->requestConnect();
     QVERIFY(mConnReady != NULL);
     QVERIFY(connect(mConnReady,
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -479,7 +498,7 @@ void TestClientFactories::initTestCase()
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
     QCOMPARE(static_cast<uint>(mConn->status()),
-             static_cast<uint>(Connection::StatusConnected));
+             static_cast<uint>(ConnectionStatusConnected));
 
     // create a Channel by magic, rather than doing D-Bus round-trips for it
 
@@ -536,7 +555,8 @@ void TestClientFactories::initTestCase()
     mClientObject1Path = QLatin1String("/org/freedesktop/Telepathy/Client/foo");
 
     ChannelDetailsList channelDetailsList;
-    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath), QVariantMap() };
+    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath),
+        ChannelClassSpec::textChat().allProperties() };
     channelDetailsList.append(channelDetails);
 
     mCDO = new ChannelDispatchOperationAdaptor(QDBusObjectPath(mAccount->objectPath()),
@@ -578,18 +598,12 @@ void TestClientFactories::testRegister()
     // invalid client
     QVERIFY(!mClientRegistrar->registerClient(AbstractClientPtr(), QLatin1String("foo")));
 
-    mClientCapabilities.append(
-        QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".MediaSignalling/ice-udp=true"));
-    mClientCapabilities.append(
-        QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".MediaSignalling/audio/speex=true"));
+    mClientCapabilities.setICEUDPNATTraversalToken();
+    mClientCapabilities.setAudioCodecToken(QLatin1String("speex"));
+
+    ChannelClassSpecList filters;
+    filters.append(ChannelClassSpec::textChat());
 
-    ChannelClassList filters;
-    QMap<QString, QDBusVariant> filter;
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
-                  QDBusVariant(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_TYPE_TEXT)));
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
-                  QDBusVariant((uint) Tp::HandleTypeContact));
-    filters.append(filter);
     mClientObject1 = MyClient::create(filters, mClientCapabilities, false, true);
     QVERIFY(mClientRegistrar->registerClient(mClientObject1, QLatin1String("foo")));
     QVERIFY(mClientRegistrar->registeredClients().contains(mClientObject1));
@@ -598,12 +612,7 @@ void TestClientFactories::testRegister()
     QVERIFY(mClientRegistrar->registerClient(mClientObject1, QLatin1String("foo")));
 
     filters.clear();
-    filter.clear();
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".ChannelType"),
-                  QDBusVariant(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_TYPE_STREAMED_MEDIA)));
-    filter.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandleType"),
-                  QDBusVariant((uint) Tp::HandleTypeContact));
-    filters.append(filter);
+    filters.append(ChannelClassSpec::streamedMediaCall());
     mClientObject2 = MyClient::create(filters, mClientCapabilities, true, true);
     QVERIFY(mClientRegistrar->registerClient(mClientObject2, QLatin1String("foo"), true));
     QVERIFY(mClientRegistrar->registeredClients().contains(mClientObject2));
@@ -630,16 +639,26 @@ void TestClientFactories::testRegister()
 void TestClientFactories::testCapabilities()
 {
     QDBusConnection bus = mClientRegistrar->dbusConnection();
+    QStringList normalizedClientCaps = mClientCapabilities.allTokens();
+    normalizedClientCaps.sort();
+
+    QStringList normalizedHandlerCaps;
 
     // object 1
     ClientHandlerInterface *handler1Iface = new ClientHandlerInterface(bus,
             mClientObject1BusName, mClientObject1Path, this);
-    QCOMPARE(handler1Iface->Capabilities(), mClientCapabilities);
+
+    QVERIFY(waitForProperty(handler1Iface->requestPropertyCapabilities(), &normalizedHandlerCaps));
+    normalizedHandlerCaps.sort();
+    QCOMPARE(normalizedHandlerCaps, normalizedClientCaps);
 
     // object 2
     ClientHandlerInterface *handler2Iface = new ClientHandlerInterface(bus,
             mClientObject2BusName, mClientObject2Path, this);
-    QCOMPARE(handler2Iface->Capabilities(), mClientCapabilities);
+
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyCapabilities(), &normalizedHandlerCaps));
+    normalizedHandlerCaps.sort();
+    QCOMPARE(normalizedHandlerCaps, normalizedClientCaps);
 }
 
 void TestClientFactories::testRequests()
@@ -652,14 +671,42 @@ void TestClientFactories::testRequests()
     connect(client,
             SIGNAL(requestAdded(const Tp::ChannelRequestPtr &)),
             SLOT(expectSignalEmission()));
-    handlerRequestsIface->AddRequest(QDBusObjectPath(mChannelRequestPath), QVariantMap());
+
+    QVariantMap requestProps;
+    requestProps.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".Account"),
+            QVariant::fromValue(QDBusObjectPath(mAccount->objectPath())));
+    requestProps.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST
+                ".Interface.DomainSpecific.IntegerProp"),
+            QVariant::fromValue(3));
+
+    handlerRequestsIface->AddRequest(QDBusObjectPath(mChannelRequestPath), requestProps);
+
     if (!client->mAddRequestRequest) {
         QCOMPARE(mLoop->exec(), 0);
     }
     QCOMPARE(client->mAddRequestRequest->objectPath(),
              mChannelRequestPath);
-//    QCOMPARE(client->mAddRequestRequest->account().data(),
-//             mAccount.data());
+    QCOMPARE(client->mAddRequestRequest->account().data(),
+             mAccount.data());
+
+    QVERIFY(client->mAddRequestRequest->immutableProperties().contains(
+                QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST
+                    ".Interface.DomainSpecific.IntegerProp")));
+    QCOMPARE(qdbus_cast<int>(client->mAddRequestRequest->immutableProperties().value(
+                    QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST
+                        ".Interface.DomainSpecific.IntegerProp"))),
+            3);
+
+    QVERIFY(connect(client->mAddRequestRequest->becomeReady(),
+                SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
+
+    QVERIFY(client->mAddRequestRequest->immutableProperties().contains(
+                QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".UserActionTime")));
+    QCOMPARE(qdbus_cast<uint>(client->mAddRequestRequest->immutableProperties().value(
+                    QLatin1String(TELEPATHY_INTERFACE_CHANNEL_REQUEST ".UserActionTime"))),
+            mUserActionTime);
 
     connect(client,
             SIGNAL(requestRemoved(const Tp::ChannelRequestPtr &,
@@ -694,7 +741,8 @@ void TestClientFactories::testObserveChannelsCommon(const AbstractClientPtr &cli
             SIGNAL(observeChannelsFinished()),
             SLOT(expectSignalEmission()));
     ChannelDetailsList channelDetailsList;
-    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath), QVariantMap() };
+    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath),
+        ChannelClassSpec::textChat().allProperties() };
     channelDetailsList.append(channelDetails);
     observeIface->ObserveChannels(QDBusObjectPath(mAccount->objectPath()),
             QDBusObjectPath(mConn->objectPath()),
@@ -795,7 +843,8 @@ void TestClientFactories::testHandleChannels()
             SIGNAL(handleChannelsFinished()),
             SLOT(expectSignalEmission()));
     ChannelDetailsList channelDetailsList;
-    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath), QVariantMap() };
+    ChannelDetails channelDetails = { QDBusObjectPath(mText1ChanPath),
+        ChannelClassSpec::textChat().allProperties() };
     channelDetailsList.append(channelDetails);
     handler1Iface->HandleChannels(QDBusObjectPath(mAccount->objectPath()),
             QDBusObjectPath(mConn->objectPath()),
@@ -816,6 +865,15 @@ void TestClientFactories::testHandleChannels()
 
     QCOMPARE(client1->mHandleChannelsChannels.first()->objectPath(), mText1ChanPath);
 
+    TextChannelPtr textChan = TextChannelPtr::qObjectCast(client1->mHandleChannelsChannels.first());
+
+    QVERIFY(!textChan.isNull());
+
+    QVERIFY(textChan->isReady());
+    QVERIFY(textChan->isReady(Channel::FeatureCore));
+    QVERIFY(textChan->isReady(TextChannel::FeatureMessageQueue));
+    QVERIFY(textChan->isReady(TextChannel::FeatureChatState));
+
     QCOMPARE(client1->mHandleChannelsRequestsSatisfied.first()->objectPath(), mChannelRequestPath);
     QVERIFY(client1->mHandleChannelsRequestsSatisfied.first()->isReady());
     QCOMPARE(client1->mHandleChannelsRequestsSatisfied.first()->account().data(),
@@ -823,7 +881,8 @@ void TestClientFactories::testHandleChannels()
 
     QCOMPARE(client1->mHandleChannelsUserActionTime.toTime_t(), mUserActionTime);
 
-    Tp::ObjectPathList handledChannels = handler1Iface->HandledChannels();
+    Tp::ObjectPathList handledChannels;
+    QVERIFY(waitForProperty(handler1Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText1ChanPath)));
 
     // object 2
@@ -862,15 +921,16 @@ void TestClientFactories::testHandleChannels()
 
     QCOMPARE(client2->mHandleChannelsUserActionTime.toTime_t(), mUserActionTime);
 
-    handledChannels = handler1Iface->HandledChannels();
+    QVERIFY(waitForProperty(handler1Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText1ChanPath)));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText2ChanPath)));
-    handledChannels = handler2Iface->HandledChannels();
+
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText1ChanPath)));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText2ChanPath)));
 
     mClientRegistrar->unregisterClient(mClientObject1);
-    handledChannels = handler2Iface->HandledChannels();
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.contains(QDBusObjectPath(mText2ChanPath)));
 
     g_object_unref(mText2ChanService);
@@ -878,7 +938,7 @@ void TestClientFactories::testHandleChannels()
             SIGNAL(channelClosed()),
             SLOT(expectSignalEmission()));
     QCOMPARE(mLoop->exec(), 0);
-    handledChannels = handler2Iface->HandledChannels();
+    QVERIFY(waitForProperty(handler2Iface->requestPropertyHandledChannels(), &handledChannels));
     QVERIFY(handledChannels.isEmpty());
 }
 
