@@ -1,5 +1,11 @@
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/AvatarData>
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Contact>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingContacts>
 #include <TelepathyQt4/PendingReady>
@@ -112,9 +118,10 @@ void TestContactsAvatar::createContactWithFakeAvatar(const char *id)
 
     tp_tests_contacts_connection_change_avatar_data(mConnService, handle,
         array, avatarMimeType, avatarToken);
+    g_array_unref(array);
 
     Tp::UIntList handles = Tp::UIntList() << handle;
-    QSet<Contact::Feature> features = QSet<Contact::Feature>()
+    Features features = Features()
         << Contact::FeatureAvatarToken
         << Contact::FeatureAvatarData;
 
@@ -166,8 +173,11 @@ void TestContactsAvatar::testAvatar()
     QByteArray a = tmpDir.toLatin1();
     setenv ("XDG_CACHE_HOME", a.constData(), true);
 
+    Client::ConnectionInterfaceAvatarsInterface *connAvatarsInterface =
+        mConn->optionalInterface<Client::ConnectionInterfaceAvatarsInterface>();
+
     /* Check if AvatarRetrieved gets called */
-    connect(mConn->avatarsInterface(),
+    connect(connAvatarsInterface,
             SIGNAL(AvatarRetrieved(uint, const QString &, const QByteArray &, const QString &)),
             SLOT(onAvatarRetrieved(uint, const QString &, const QByteArray &, const QString &)));
 
@@ -251,16 +261,18 @@ void TestContactsAvatar::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
 
-    QCOMPARE(mConn->status(), Connection::StatusConnected);
+    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 
     QVERIFY(mConn->contactManager()->supportedFeatures().contains(Contact::FeatureAvatarData));
 }
@@ -279,7 +291,7 @@ void TestContactsAvatar::cleanupTestCase()
 {
     if (mConn) {
         // Disconnect and wait for invalidated
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
