@@ -1,5 +1,11 @@
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
+#include <TelepathyQt4/Contact>
 #include <TelepathyQt4/ContactCapabilities>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingContacts>
 #include <TelepathyQt4/PendingReady>
@@ -105,16 +111,18 @@ void TestContactsCapabilities::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
 
-    QCOMPARE(mConn->status(), Connection::StatusConnected);
+    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 
     QVERIFY(mConn->contactManager()->supportedFeatures().contains(Contact::FeatureCapabilities));
 }
@@ -186,9 +194,10 @@ void TestContactsCapabilities::testCapabilities()
 
     GHashTable *capabilities = createContactCapabilities(handles);
     tp_tests_contacts_connection_change_capabilities(mConnService, capabilities);
+    g_hash_table_destroy(capabilities);
 
     PendingContacts *pending = mConn->contactManager()->contactsForIdentifiers(
-            ids, QSet<Contact::Feature>() << Contact::FeatureCapabilities);
+            ids, Features() << Contact::FeatureCapabilities);
     QVERIFY(connect(pending,
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectPendingContactsFinished(Tp::PendingOperation*))));
@@ -197,18 +206,15 @@ void TestContactsCapabilities::testCapabilities()
     for (int i = 0; i < mContacts.size(); i++) {
         ContactPtr contact = mContacts[i];
 
-        QCOMPARE(contact->requestedFeatures(),
-                 QSet<Contact::Feature>() << Contact::FeatureCapabilities);
-        QCOMPARE(contact->actualFeatures(),
-                 QSet<Contact::Feature>() << Contact::FeatureCapabilities);
+        QCOMPARE(contact->requestedFeatures().contains(Contact::FeatureCapabilities), true);
+        QCOMPARE(contact->actualFeatures().contains(Contact::FeatureCapabilities), true);
 
-        QVERIFY(contact->capabilities() != 0);
-        QCOMPARE(contact->capabilities()->supportsTextChats(), supportTextChat[i]);
-        QCOMPARE(contact->capabilities()->supportsMediaCalls(), false);
-        QCOMPARE(contact->capabilities()->supportsAudioCalls(), false);
-        QCOMPARE(contact->capabilities()->supportsVideoCalls(false), false);
-        QCOMPARE(contact->capabilities()->supportsVideoCalls(true), false);
-        QCOMPARE(contact->capabilities()->supportsUpgradingCalls(), false);
+        QCOMPARE(contact->capabilities().textChats(), supportTextChat[i]);
+        QCOMPARE(contact->capabilities().streamedMediaCalls(), false);
+        QCOMPARE(contact->capabilities().streamedMediaAudioCalls(), false);
+        QCOMPARE(contact->capabilities().streamedMediaVideoCalls(), false);
+        QCOMPARE(contact->capabilities().streamedMediaVideoCallsWithAudio(), false);
+        QCOMPARE(contact->capabilities().upgradingStreamedMediaCalls(), false);
     }
 }
 
@@ -221,7 +227,7 @@ void TestContactsCapabilities::cleanupTestCase()
 {
     if (mConn) {
         // Disconnect and wait for invalidated
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
