@@ -1,5 +1,10 @@
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Contact>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingContacts>
 #include <TelepathyQt4/PendingContactInfo>
@@ -24,7 +29,7 @@ public:
 protected Q_SLOTS:
     void expectConnInvalidated();
     void expectPendingContactsFinished(Tp::PendingOperation *);
-    void onContactInfoChanged(const Tp::ContactInfoFieldList &);
+    void onContactInfoFieldsChanged(const Tp::Contact::InfoFields &);
 
 private Q_SLOTS:
     void initTestCase();
@@ -40,7 +45,7 @@ private:
     TpTestsContactsConnection *mConnService;
     ConnectionPtr mConn;
     QList<ContactPtr> mContacts;
-    int mContactsInfoUpdated;
+    int mContactsInfoFieldsUpdated;
 };
 
 void TestContactsInfo::expectConnInvalidated()
@@ -76,10 +81,10 @@ void TestContactsInfo::expectPendingContactsFinished(PendingOperation *op)
     mLoop->exit(0);
 }
 
-void TestContactsInfo::onContactInfoChanged(const Tp::ContactInfoFieldList &info)
+void TestContactsInfo::onContactInfoFieldsChanged(const Tp::Contact::InfoFields &info)
 {
     Q_UNUSED(info);
-    mContactsInfoUpdated++;
+    mContactsInfoFieldsUpdated++;
     mLoop->exit(0);
 }
 
@@ -115,16 +120,18 @@ void TestContactsInfo::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    QVERIFY(connect(mConn->requestConnect(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
 
-    QCOMPARE(mConn->status(), Connection::StatusConnected);
+    QCOMPARE(mConn->status(), ConnectionStatusConnected);
 
     QVERIFY(mConn->contactManager()->supportedFeatures().contains(Contact::FeatureInfo));
 }
@@ -132,7 +139,7 @@ void TestContactsInfo::initTestCase()
 void TestContactsInfo::init()
 {
     initImpl();
-    mContactsInfoUpdated = 0;
+    mContactsInfoFieldsUpdated = 0;
 }
 
 void TestContactsInfo::testInfo()
@@ -141,7 +148,7 @@ void TestContactsInfo::testInfo()
         << QLatin1String("bar");
 
     PendingContacts *pending = mConn->contactManager()->contactsForIdentifiers(
-            validIDs, QSet<Contact::Feature>() << Contact::FeatureInfo);
+            validIDs, Features() << Contact::FeatureInfo);
     QVERIFY(connect(pending,
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectPendingContactsFinished(Tp::PendingOperation*))));
@@ -150,16 +157,14 @@ void TestContactsInfo::testInfo()
     for (int i = 0; i < mContacts.size(); i++) {
         ContactPtr contact = mContacts[i];
 
-        QCOMPARE(contact->requestedFeatures(),
-                 QSet<Contact::Feature>() << Contact::FeatureInfo);
-        QCOMPARE(contact->actualFeatures(),
-                 QSet<Contact::Feature>() << Contact::FeatureInfo);
+        QCOMPARE(contact->requestedFeatures().contains(Contact::FeatureInfo), true);
+        QCOMPARE(contact->actualFeatures().contains(Contact::FeatureInfo), true);
 
-        QVERIFY(contact->info().isEmpty());
+        QVERIFY(contact->infoFields().allFields().isEmpty());
 
-        connect(contact.data(),
-                SIGNAL(infoChanged(const Tp::ContactInfoFieldList &)),
-                SLOT(onContactInfoChanged(const Tp::ContactInfoFieldList &)));
+        QVERIFY(connect(contact.data(),
+                        SIGNAL(infoFieldsChanged(const Tp::Contact::InfoFields &)),
+                        SLOT(onContactInfoFieldsChanged(const Tp::Contact::InfoFields &))));
     }
 
     GPtrArray *info_1 = (GPtrArray *) dbus_g_type_specialized_construct (
@@ -199,20 +204,24 @@ void TestContactsInfo::testInfo()
     tp_tests_contacts_connection_change_contact_info(mConnService, handles[0], info_1);
     tp_tests_contacts_connection_change_contact_info(mConnService, handles[1], info_2);
 
-    while (mContactsInfoUpdated != 2)  {
+    while (mContactsInfoFieldsUpdated != 2)  {
         QCOMPARE(mLoop->exec(), 0);
     }
 
-    mContactsInfoUpdated = 0;
+    QCOMPARE(mContactsInfoFieldsUpdated, 2);
+
+    mContactsInfoFieldsUpdated = 0;
     ContactPtr contactFoo = mContacts[0];
     ContactPtr contactBar = mContacts[1];
 
-    QCOMPARE(contactFoo->info().size(), 1);
-    QCOMPARE(contactFoo->info()[0].fieldName, QLatin1String("n"));
-    QCOMPARE(contactFoo->info()[0].fieldValue[0], QLatin1String("Foo"));
-    QCOMPARE(contactBar->info().size(), 1);
-    QCOMPARE(contactBar->info()[0].fieldName, QLatin1String("n"));
-    QCOMPARE(contactBar->info()[0].fieldValue[0], QLatin1String("Bar"));
+    QCOMPARE(contactFoo->infoFields().isValid(), true);
+    QCOMPARE(contactFoo->infoFields().allFields().size(), 1);
+    QCOMPARE(contactFoo->infoFields().allFields()[0].fieldName, QLatin1String("n"));
+    QCOMPARE(contactFoo->infoFields().allFields()[0].fieldValue[0], QLatin1String("Foo"));
+    QCOMPARE(contactBar->infoFields().isValid(), true);
+    QCOMPARE(contactBar->infoFields().allFields().size(), 1);
+    QCOMPARE(contactBar->infoFields().allFields()[0].fieldName, QLatin1String("n"));
+    QCOMPARE(contactBar->infoFields().allFields()[0].fieldValue[0], QLatin1String("Bar"));
 
     Q_FOREACH (const ContactPtr &contact, mContacts) {
         PendingOperation *op = contact->refreshInfo();
@@ -223,7 +232,7 @@ void TestContactsInfo::testInfo()
     }
 
     /* nothing changed */
-    QCOMPARE(mContactsInfoUpdated, 0);
+    QCOMPARE(mContactsInfoFieldsUpdated, 0);
 
     for (int i = 0; i < mContacts.size(); i++) {
         ContactPtr contact = mContacts[i];
@@ -241,12 +250,15 @@ void TestContactsInfo::testInfo()
         QCOMPARE(mLoop->exec(), 0);
     }
 
-    QCOMPARE(pci->info().size(), 1);
-    QCOMPARE(pci->info()[0].fieldName, QLatin1String("n"));
-    QCOMPARE(pci->info()[0].fieldValue[0], QLatin1String("Foo"));
+    QCOMPARE(pci->infoFields().isValid(), true);
+    QCOMPARE(pci->infoFields().allFields().size(), 1);
+    QCOMPARE(pci->infoFields().allFields()[0].fieldName, QLatin1String("n"));
+    QCOMPARE(pci->infoFields().allFields()[0].fieldValue[0], QLatin1String("Foo"));
 
     g_boxed_free (TP_ARRAY_TYPE_CONTACT_INFO_FIELD_LIST, info_1);
+    g_ptr_array_unref (info_1);
     g_boxed_free (TP_ARRAY_TYPE_CONTACT_INFO_FIELD_LIST, info_2);
+    g_ptr_array_unref (info_2);
 }
 
 void TestContactsInfo::cleanup()
@@ -258,7 +270,7 @@ void TestContactsInfo::cleanupTestCase()
 {
     if (mConn) {
         // Disconnect and wait for invalidated
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
