@@ -5,12 +5,18 @@
 #include <QtDBus>
 #include <QtTest>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
 #include <TelepathyQt4/Contact>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingContacts>
 #include <TelepathyQt4/PendingVoid>
 #include <TelepathyQt4/PendingReady>
+#include <TelepathyQt4/Presence>
 #include <TelepathyQt4/ReferencedHandles>
 #include <TelepathyQt4/Debug>
 #include <TelepathyQt4/Types>
@@ -34,7 +40,7 @@ public:
     }
 
 protected Q_SLOTS:
-    void expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason);
+    void expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason);
     void expectConnInvalidated();
     void expectPendingContactsFinished(Tp::PendingOperation *);
 
@@ -62,18 +68,18 @@ private:
     Tp::UIntList mInvalidHandles;
 };
 
-void TestContacts::expectConnReady(Tp::Connection::Status newStatus,
+void TestContacts::expectConnReady(Tp::ConnectionStatus newStatus,
         Tp::ConnectionStatusReason newStatusReason)
 {
     switch (newStatus) {
-    case Connection::StatusDisconnected:
+    case ConnectionStatusDisconnected:
         qWarning() << "Disconnected";
         mLoop->exit(1);
         break;
-    case Connection::StatusConnecting:
+    case ConnectionStatusConnecting:
         /* do nothing */
         break;
-    case Connection::StatusConnected:
+    case ConnectionStatusConnected:
         qDebug() << "Ready";
         mLoop->exit(0);
         break;
@@ -154,10 +160,12 @@ void TestContacts::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    mConn->requestConnect();
+    mConn->lowlevel()->requestConnect();
 
     Features features = Features() << Connection::FeatureSelfContact;
     QVERIFY(connect(mConn->becomeReady(features),
@@ -166,16 +174,16 @@ void TestContacts::initTestCase()
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(features), true);
 
-    if (mConn->status() != Connection::StatusConnected) {
+    if (mConn->status() != ConnectionStatusConnected) {
         QVERIFY(connect(mConn.data(),
-                        SIGNAL(statusChanged(Tp::Connection::Status, Tp::ConnectionStatusReason)),
-                        SLOT(expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason))));
+                        SIGNAL(statusChanged(Tp::ConnectionStatus, Tp::ConnectionStatusReason)),
+                        SLOT(expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason))));
         QCOMPARE(mLoop->exec(), 0);
         QVERIFY(disconnect(mConn.data(),
-                           SIGNAL(statusChanged(Tp::Connection::Status, Tp::ConnectionStatusReason)),
+                           SIGNAL(statusChanged(Tp::ConnectionStatus, Tp::ConnectionStatusReason)),
                            this,
-                           SLOT(expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason))));
-        QCOMPARE(mConn->status(), Connection::StatusConnected);
+                           SLOT(expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason))));
+        QCOMPARE(mConn->status(), ConnectionStatusConnected);
     }
 }
 
@@ -188,18 +196,18 @@ void TestContacts::testSupport()
 {
     QCOMPARE(mConn->contactManager()->connection(), mConn);
 
-    QVERIFY(!mConn->contactAttributeInterfaces().isEmpty());
+    QVERIFY(!mConn->lowlevel()->contactAttributeInterfaces().isEmpty());
 
-    QVERIFY(mConn->contactAttributeInterfaces().contains(
+    QVERIFY(mConn->lowlevel()->contactAttributeInterfaces().contains(
                 QLatin1String(TELEPATHY_INTERFACE_CONNECTION)));
-    QVERIFY(mConn->contactAttributeInterfaces().contains(
+    QVERIFY(mConn->lowlevel()->contactAttributeInterfaces().contains(
                 QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_ALIASING)));
-    QVERIFY(mConn->contactAttributeInterfaces().contains(
+    QVERIFY(mConn->lowlevel()->contactAttributeInterfaces().contains(
                 QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_AVATARS)));
-    QVERIFY(mConn->contactAttributeInterfaces().contains(
+    QVERIFY(mConn->lowlevel()->contactAttributeInterfaces().contains(
                 QLatin1String(TELEPATHY_INTERFACE_CONNECTION_INTERFACE_SIMPLE_PRESENCE)));
 
-    QSet<Contact::Feature> supportedFeatures = mConn->contactManager()->supportedFeatures();
+    Features supportedFeatures = mConn->contactManager()->supportedFeatures();
     QVERIFY(!supportedFeatures.isEmpty());
     QVERIFY(supportedFeatures.contains(Contact::FeatureAlias));
     QVERIFY(supportedFeatures.contains(Contact::FeatureAvatarToken));
@@ -216,7 +224,7 @@ void TestContacts::testSelfContact()
 
     QVERIFY(connect(selfContact->manager()->upgradeContacts(
                         QList<ContactPtr>() << selfContact,
-                        QSet<Contact::Feature>() << Contact::FeatureAlias
+                        Features() << Contact::FeatureAlias
                             << Contact::FeatureAvatarToken
                             << Contact::FeatureSimplePresence),
                     SIGNAL(finished(Tp::PendingOperation*)),
@@ -227,9 +235,9 @@ void TestContacts::testSelfContact()
 
     QVERIFY(!selfContact->isAvatarTokenKnown());
 
-    QCOMPARE(selfContact->presenceStatus(), QString(QLatin1String("available")));
-    QCOMPARE(selfContact->presenceType(), uint(Tp::ConnectionPresenceTypeAvailable));
-    QCOMPARE(selfContact->presenceMessage(), QString(QLatin1String("")));
+    QCOMPARE(selfContact->presence().status(), QString(QLatin1String("available")));
+    QCOMPARE(selfContact->presence().type(), Tp::ConnectionPresenceTypeAvailable);
+    QCOMPARE(selfContact->presence().statusMessage(), QString(QLatin1String("")));
 }
 
 void TestContacts::testForHandles()
@@ -258,7 +266,7 @@ void TestContacts::testForHandles()
 
     // Test the closure accessors
     QCOMPARE(pending->manager(), mConn->contactManager());
-    QCOMPARE(pending->features(), QSet<Contact::Feature>());
+    QCOMPARE(pending->features(), Features());
 
     QVERIFY(pending->isForHandles());
     QVERIFY(!pending->isForIdentifiers());
@@ -283,8 +291,8 @@ void TestContacts::testForHandles()
     for (int i = 0; i < 3; i++) {
         QVERIFY(mContacts[i] != NULL);
         QCOMPARE(mContacts[i]->manager(), mConn->contactManager());
-        QCOMPARE(mContacts[i]->requestedFeatures(), QSet<Contact::Feature>());
-        QCOMPARE(mContacts[i]->actualFeatures(), QSet<Contact::Feature>());
+        QCOMPARE(mContacts[i]->requestedFeatures(), Features());
+        QCOMPARE(mContacts[i]->actualFeatures(), Features());
     }
 
     QCOMPARE(mContacts[0]->handle()[0], handles[0]);
@@ -325,16 +333,6 @@ void TestContacts::testForHandles()
     mContacts.clear();
     mLoop->processEvents();
     processDBusQueue(mConn.data());
-
-    // Unref the handles we created service-side
-    tp_handle_unref(serviceRepo, handles[0]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[0], NULL));
-    tp_handle_unref(serviceRepo, handles[1]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[1], NULL));
-    tp_handle_unref(serviceRepo, handles[2]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[2], NULL));
-    tp_handle_unref(serviceRepo, handles[3]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[3], NULL));
 }
 
 void TestContacts::testForIdentifiers()
@@ -376,7 +374,7 @@ void TestContacts::testForIdentifiers()
 
     // Test the closure accessors
     QCOMPARE(pending->manager(), mConn->contactManager());
-    QCOMPARE(pending->features(), QSet<Contact::Feature>());
+    QCOMPARE(pending->features(), Features());
 
     QVERIFY(!pending->isForHandles());
     QVERIFY(pending->isForIdentifiers());
@@ -397,8 +395,8 @@ void TestContacts::testForIdentifiers()
         QVERIFY(mContacts[i] != NULL);
         QCOMPARE(mContacts[i]->manager(), mConn->contactManager());
         QVERIFY(tp_handle_is_valid(serviceRepo, mContacts[i]->handle()[0], NULL));
-        QCOMPARE(mContacts[i]->requestedFeatures(), QSet<Contact::Feature>());
-        QCOMPARE(mContacts[i]->actualFeatures(), QSet<Contact::Feature>());
+        QCOMPARE(mContacts[i]->requestedFeatures(), Features());
+        QCOMPARE(mContacts[i]->actualFeatures(), Features());
     }
 
     QCOMPARE(mContacts[0]->id(), QString(QLatin1String("alice")));
@@ -413,11 +411,6 @@ void TestContacts::testForIdentifiers()
     mContacts.clear();
     mLoop->processEvents();
     processDBusQueue(mConn.data());
-
-    // Check that their handles are in fact released
-    Q_FOREACH (uint handle, saveHandles) {
-        QVERIFY(!tp_handle_is_valid(serviceRepo, handle, NULL));
-    }
 }
 
 void TestContacts::testFeatures()
@@ -459,7 +452,7 @@ void TestContacts::testFeatures()
         "Having some carrots",
         "Done building for life, yay",
     };
-    QSet<Contact::Feature> features = QSet<Contact::Feature>()
+    Features features = Features()
         << Contact::FeatureAlias
         << Contact::FeatureAvatarToken
         << Contact::FeatureSimplePresence;
@@ -501,7 +494,7 @@ void TestContacts::testFeatures()
 
         QVERIFY(mContacts[i]->actualFeatures().contains(Contact::FeatureAvatarToken));
         QVERIFY(mContacts[i]->actualFeatures().contains(Contact::FeatureSimplePresence));
-        QCOMPARE(mContacts[i]->presenceMessage(), QString(QLatin1String(initialMessages[i])));
+        QCOMPARE(mContacts[i]->presence().statusMessage(), QString(QLatin1String(initialMessages[i])));
     }
 
     // Check that there's no known avatar token for the first contact, but that there is for the
@@ -514,13 +507,13 @@ void TestContacts::testFeatures()
     QCOMPARE(mContacts[1]->avatarToken(), QString(QLatin1String(initialTokens[0])));
     QCOMPARE(mContacts[2]->avatarToken(), QString(QLatin1String(initialTokens[1])));
 
-    QCOMPARE(mContacts[0]->presenceStatus(), QString(QLatin1String("available")));
-    QCOMPARE(mContacts[1]->presenceStatus(), QString(QLatin1String("busy")));
-    QCOMPARE(mContacts[2]->presenceStatus(), QString(QLatin1String("away")));
+    QCOMPARE(mContacts[0]->presence().status(), QString(QLatin1String("available")));
+    QCOMPARE(mContacts[1]->presence().status(), QString(QLatin1String("busy")));
+    QCOMPARE(mContacts[2]->presence().status(), QString(QLatin1String("away")));
 
-    QCOMPARE(mContacts[0]->presenceType(), uint(Tp::ConnectionPresenceTypeAvailable));
-    QCOMPARE(mContacts[1]->presenceType(), uint(Tp::ConnectionPresenceTypeBusy));
-    QCOMPARE(mContacts[2]->presenceType(), uint(Tp::ConnectionPresenceTypeAway));
+    QCOMPARE(mContacts[0]->presence().type(), Tp::ConnectionPresenceTypeAvailable);
+    QCOMPARE(mContacts[1]->presence().type(), Tp::ConnectionPresenceTypeBusy);
+    QCOMPARE(mContacts[2]->presence().type(), Tp::ConnectionPresenceTypeAway);
 
     // Change some of the contacts to a new set of attributes
     tp_tests_contacts_connection_change_aliases(mConnService, 2, handles.toVector().constData(),
@@ -554,30 +547,22 @@ void TestContacts::testFeatures()
     QCOMPARE(mContacts[1]->avatarToken(), QString(QLatin1String(latterTokens[1])));
     QCOMPARE(mContacts[2]->avatarToken(), QString(QLatin1String(initialTokens[1])));
 
-    QCOMPARE(mContacts[0]->presenceStatus(), QString(QLatin1String("away")));
-    QCOMPARE(mContacts[1]->presenceStatus(), QString(QLatin1String("available")));
-    QCOMPARE(mContacts[2]->presenceStatus(), QString(QLatin1String("away")));
+    QCOMPARE(mContacts[0]->presence().status(), QString(QLatin1String("away")));
+    QCOMPARE(mContacts[1]->presence().status(), QString(QLatin1String("available")));
+    QCOMPARE(mContacts[2]->presence().status(), QString(QLatin1String("away")));
 
-    QCOMPARE(mContacts[0]->presenceType(), uint(Tp::ConnectionPresenceTypeAway));
-    QCOMPARE(mContacts[1]->presenceType(), uint(Tp::ConnectionPresenceTypeAvailable));
-    QCOMPARE(mContacts[2]->presenceType(), uint(Tp::ConnectionPresenceTypeAway));
+    QCOMPARE(mContacts[0]->presence().type(), Tp::ConnectionPresenceTypeAway);
+    QCOMPARE(mContacts[1]->presence().type(), Tp::ConnectionPresenceTypeAvailable);
+    QCOMPARE(mContacts[2]->presence().type(), Tp::ConnectionPresenceTypeAway);
 
-    QCOMPARE(mContacts[0]->presenceMessage(), QString(QLatin1String(latterMessages[0])));
-    QCOMPARE(mContacts[1]->presenceMessage(), QString(QLatin1String(latterMessages[1])));
-    QCOMPARE(mContacts[2]->presenceMessage(), QString(QLatin1String(initialMessages[2])));
+    QCOMPARE(mContacts[0]->presence().statusMessage(), QString(QLatin1String(latterMessages[0])));
+    QCOMPARE(mContacts[1]->presence().statusMessage(), QString(QLatin1String(latterMessages[1])));
+    QCOMPARE(mContacts[2]->presence().statusMessage(), QString(QLatin1String(initialMessages[2])));
 
     // Make the contacts go out of scope, starting releasing their handles, and finish that
     mContacts.clear();
     mLoop->processEvents();
     processDBusQueue(mConn.data());
-
-    // Unref the handles we created service-side
-    tp_handle_unref(serviceRepo, handles[0]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[0], NULL));
-    tp_handle_unref(serviceRepo, handles[1]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[1], NULL));
-    tp_handle_unref(serviceRepo, handles[2]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[2], NULL));
 }
 
 void TestContacts::testFeaturesNotRequested()
@@ -614,23 +599,13 @@ void TestContacts::testFeaturesNotRequested()
         QVERIFY(!contact->isAvatarTokenKnown());
         QCOMPARE(contact->avatarToken(), QString(QLatin1String("")));
 
-        QCOMPARE(contact->presenceStatus(), QString(QLatin1String("unknown")));
-        QCOMPARE(contact->presenceType(), uint(Tp::ConnectionPresenceTypeUnknown));
-        QCOMPARE(contact->presenceMessage(), QString(QLatin1String("")));
+        QCOMPARE(contact->presence().isValid(), false);
     }
 
     // Make the contacts go out of scope, starting releasing their handles, and finish that
     mContacts.clear();
     mLoop->processEvents();
     processDBusQueue(mConn.data());
-
-    // Unref the handles we created service-side
-    tp_handle_unref(serviceRepo, handles[0]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[0], NULL));
-    tp_handle_unref(serviceRepo, handles[1]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[1], NULL));
-    tp_handle_unref(serviceRepo, handles[2]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[2], NULL));
 }
 
 void TestContacts::testUpgrade()
@@ -684,7 +659,7 @@ void TestContacts::testUpgrade()
     QList<ContactPtr> saveContacts = mContacts;
 
     // Upgrade them
-    QSet<Contact::Feature> features = QSet<Contact::Feature>()
+    Features features = Features()
         << Contact::FeatureAlias
         << Contact::FeatureAvatarToken
         << Contact::FeatureSimplePresence;
@@ -724,30 +699,22 @@ void TestContacts::testUpgrade()
         QCOMPARE(mContacts[i]->avatarToken(), QString(QLatin1String(tokens[i])));
 
         QVERIFY(mContacts[i]->actualFeatures().contains(Contact::FeatureSimplePresence));
-        QCOMPARE(mContacts[i]->presenceMessage(), QString(QLatin1String(messages[i])));
+        QCOMPARE(mContacts[i]->presence().statusMessage(), QString(QLatin1String(messages[i])));
     }
 
-    QCOMPARE(mContacts[0]->presenceStatus(), QString(QLatin1String("available")));
-    QCOMPARE(mContacts[1]->presenceStatus(), QString(QLatin1String("busy")));
-    QCOMPARE(mContacts[2]->presenceStatus(), QString(QLatin1String("away")));
+    QCOMPARE(mContacts[0]->presence().status(), QString(QLatin1String("available")));
+    QCOMPARE(mContacts[1]->presence().status(), QString(QLatin1String("busy")));
+    QCOMPARE(mContacts[2]->presence().status(), QString(QLatin1String("away")));
 
-    QCOMPARE(mContacts[0]->presenceType(), uint(Tp::ConnectionPresenceTypeAvailable));
-    QCOMPARE(mContacts[1]->presenceType(), uint(Tp::ConnectionPresenceTypeBusy));
-    QCOMPARE(mContacts[2]->presenceType(), uint(Tp::ConnectionPresenceTypeAway));
+    QCOMPARE(mContacts[0]->presence().type(), Tp::ConnectionPresenceTypeAvailable);
+    QCOMPARE(mContacts[1]->presence().type(), Tp::ConnectionPresenceTypeBusy);
+    QCOMPARE(mContacts[2]->presence().type(), Tp::ConnectionPresenceTypeAway);
 
     // Make the contacts go out of scope, starting releasing their handles, and finish that
     saveContacts.clear();
     mContacts.clear();
     mLoop->processEvents();
     processDBusQueue(mConn.data());
-
-    // Unref the handles we created service-side
-    tp_handle_unref(serviceRepo, handles[0]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[0], NULL));
-    tp_handle_unref(serviceRepo, handles[1]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[1], NULL));
-    tp_handle_unref(serviceRepo, handles[2]);
-    QVERIFY(!tp_handle_is_valid(serviceRepo, handles[2], NULL));
 }
 
 void TestContacts::testSelfContactFallback()
@@ -770,16 +737,16 @@ void TestContacts::testSelfContactFallback()
     QVERIFY(name != 0);
     QVERIFY(connPath != 0);
 
-    ConnectionPtr conn = Connection::create(QLatin1String(name), QLatin1String(connPath));
+    ConnectionPtr conn = Connection::create(QLatin1String(name), QLatin1String(connPath),
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     g_free(name);
     g_free(connPath);
 
     QCOMPARE(conn->isReady(), false);
 
-    conn->requestConnect();
-
     Features features = Features() << Connection::FeatureSelfContact;
-    QVERIFY(connect(conn->becomeReady(features),
+    QVERIFY(connect(conn->lowlevel()->requestConnect(features),
                 SIGNAL(finished(Tp::PendingOperation*)),
                 SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -792,7 +759,7 @@ void TestContacts::testSelfContactFallback()
     QCOMPARE(selfContact->id(), QString(QLatin1String("me@example.com")));
     QCOMPARE(selfContact->alias(), QString(QLatin1String("me@example.com")));
     QVERIFY(!selfContact->isAvatarTokenKnown());
-    QCOMPARE(selfContact->presenceStatus(), QString(QLatin1String("unknown")));
+    QCOMPARE(selfContact->presence().isValid(), false);
 
     tp_tests_simple_connection_inject_disconnect(connService);
 
@@ -825,7 +792,7 @@ void TestContacts::cleanupTestCase()
 
     if (mConn) {
         // Disconnect and wait for the readiness change
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
