@@ -133,6 +133,7 @@ void TestDBusProxyFactory::testCaching()
             ContactFactory::create());
 
     QVERIFY(first != NULL);
+    QVERIFY(!first->object().isNull());
     QVERIFY(!first->proxy().isNull());
 
     PendingReady *same = mFactory->proxy(mConnName1, mConnPath1,
@@ -140,8 +141,10 @@ void TestDBusProxyFactory::testCaching()
             ContactFactory::create());
 
     QVERIFY(same != NULL);
+    QVERIFY(!same->object().isNull());
     QVERIFY(!same->proxy().isNull());
 
+    QCOMPARE(same->object().data(), first->object().data());
     QCOMPARE(same->proxy().data(), first->proxy().data());
 
     PendingReady *different = mFactory->proxy(mConnName2, mConnPath2,
@@ -149,11 +152,13 @@ void TestDBusProxyFactory::testCaching()
             ContactFactory::create());
 
     QVERIFY(different != NULL);
+    QVERIFY(!different->object().isNull());
     QVERIFY(!different->proxy().isNull());
 
+    QVERIFY(different->object() == first->object());
     QVERIFY(different->proxy() != first->proxy());
 
-    ConnectionPtr firstProxy = ConnectionPtr::dynamicCast(first->proxy());
+    ConnectionPtr firstProxy = ConnectionPtr::qObjectCast(first->proxy());
 
     QVERIFY(!first->isFinished() && !same->isFinished() && !different->isFinished());
     QVERIFY(connect(first, SIGNAL(finished(Tp::PendingOperation*)),
@@ -175,6 +180,7 @@ void TestDBusProxyFactory::testCaching()
             ContactFactory::create());
 
     QVERIFY(another != NULL);
+    QVERIFY(!another->object().isNull());
     QVERIFY(!another->proxy().isNull());
 
     // Should still be the same even if all the initial requests already finished
@@ -192,9 +198,10 @@ void TestDBusProxyFactory::testDropRefs()
             ContactFactory::create());
 
     QVERIFY(first != NULL);
+    QVERIFY(!first->object().isNull());
     QVERIFY(!first->proxy().isNull());
-    ConnectionPtr firstProxy = ConnectionPtr::dynamicCast(first->proxy());
 
+    ConnectionPtr firstProxy = ConnectionPtr::qObjectCast(first->proxy());
     QVERIFY(firstProxy->isValid());
 
     QVERIFY(connect(first, SIGNAL(finished(Tp::PendingOperation*)),
@@ -206,6 +213,7 @@ void TestDBusProxyFactory::testDropRefs()
             ContactFactory::create());
 
     QVERIFY(same != NULL);
+    QVERIFY(!same->object().isNull());
     QVERIFY(!same->proxy().isNull());
 
     // The first one is in scope so we should've got it again
@@ -227,6 +235,7 @@ void TestDBusProxyFactory::testDropRefs()
             ContactFactory::create());
 
     QVERIFY(different != NULL);
+    QVERIFY(!different->object().isNull());
     QVERIFY(!different->proxy().isNull());
 
     // The first one has gone out of scope and deleted so we should've got a different one
@@ -240,9 +249,10 @@ void TestDBusProxyFactory::testInvalidate()
             ContactFactory::create());
 
     QVERIFY(first != NULL);
+    QVERIFY(!first->object().isNull());
     QVERIFY(!first->proxy().isNull());
-    ConnectionPtr firstProxy = ConnectionPtr::dynamicCast(first->proxy());
 
+    ConnectionPtr firstProxy = ConnectionPtr::qObjectCast(first->proxy());
     QVERIFY(firstProxy->isValid());
 
     QVERIFY(connect(first, SIGNAL(finished(Tp::PendingOperation*)),
@@ -254,6 +264,7 @@ void TestDBusProxyFactory::testInvalidate()
             ContactFactory::create());
 
     QVERIFY(same != NULL);
+    QVERIFY(!same->object().isNull());
     QVERIFY(!same->proxy().isNull());
 
     // The first one is in scope and valid so we should've got it again
@@ -280,7 +291,7 @@ void TestDBusProxyFactory::testInvalidate()
             ContactFactory::create());
 
     QVERIFY(different != NULL);
-    ConnectionPtr differentProxy = ConnectionPtr::dynamicCast(different->proxy());
+    ConnectionPtr differentProxy = ConnectionPtr::qObjectCast(different->proxy());
     QVERIFY(!differentProxy.isNull());
 
     // The first one is invalid so we should've got a different one
@@ -306,9 +317,10 @@ void TestDBusProxyFactory::testBogusService()
             ContactFactory::create());
 
     QVERIFY(bogus != NULL);
+    QVERIFY(!bogus->object().isNull());
     QVERIFY(!bogus->proxy().isNull());
 
-    QVERIFY(!ConnectionPtr::dynamicCast(bogus->proxy())->isValid());
+    QVERIFY(!ConnectionPtr::qObjectCast(bogus->proxy())->isValid());
 
     PendingReady *another = mFactory->proxy(QLatin1String("org.bogus.Totally"),
             QLatin1String("/org/bogus/Totally"),
@@ -316,14 +328,21 @@ void TestDBusProxyFactory::testBogusService()
             ContactFactory::create());
 
     QVERIFY(another != NULL);
+    QVERIFY(!another->object().isNull());
     QVERIFY(!another->proxy().isNull());
 
-    QVERIFY(!ConnectionPtr::dynamicCast(another->proxy())->isValid());
+    QVERIFY(!ConnectionPtr::qObjectCast(another->proxy())->isValid());
 
     // We shouldn't get the same proxy twice ie. a proxy should not be cached if not present on bus
     // and invalidated because of that, otherwise we'll return an invalid instance from the cache
     // even if after the service appears on the bus
+    QCOMPARE(another->object(), bogus->object());
     QVERIFY(another->proxy() != bogus->proxy());
+
+    // The PendingReady itself should finish with failure
+    QVERIFY(connect(another, SIGNAL(finished(Tp::PendingOperation*)),
+                SLOT(expectFailure(Tp::PendingOperation*))));
+    QCOMPARE(mLoop->exec(), 0);
 }
 
 void TestDBusProxyFactory::cleanup()
