@@ -5,7 +5,12 @@
 
 #include <QtTest/QtTest>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/PendingHandles>
 #include <TelepathyQt4/PendingVoid>
 #include <TelepathyQt4/PendingReady>
@@ -29,7 +34,7 @@ public:
     { }
 
 protected Q_SLOTS:
-    void expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason);
+    void expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason);
     void expectConnInvalidated();
     void expectPendingHandlesFinished(Tp::PendingOperation*);
 
@@ -49,18 +54,18 @@ private:
     ReferencedHandles mHandles;
 };
 
-void TestHandles::expectConnReady(Tp::Connection::Status newStatus,
+void TestHandles::expectConnReady(Tp::ConnectionStatus newStatus,
         Tp::ConnectionStatusReason newStatusReason)
 {
     switch (newStatus) {
-    case Connection::StatusDisconnected:
+    case ConnectionStatusDisconnected:
         qWarning() << "Disconnected";
         mLoop->exit(1);
         break;
-    case Connection::StatusConnecting:
+    case ConnectionStatusConnecting:
         /* do nothing */
         break;
-    case Connection::StatusConnected:
+    case ConnectionStatusConnected:
         qDebug() << "Ready";
         mLoop->exit(0);
         break;
@@ -136,27 +141,27 @@ void TestHandles::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    mConn->requestConnect();
-
-    QVERIFY(connect(mConn->becomeReady(),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
 
-    if (mConn->status() != Connection::StatusConnected) {
+    if (mConn->status() != ConnectionStatusConnected) {
         QVERIFY(connect(mConn.data(),
-                        SIGNAL(statusChanged(Tp::Connection::Status, Tp::ConnectionStatusReason)),
-                        SLOT(expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason))));
+                        SIGNAL(statusChanged(Tp::ConnectionStatus, Tp::ConnectionStatusReason)),
+                        SLOT(expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason))));
         QCOMPARE(mLoop->exec(), 0);
         QVERIFY(disconnect(mConn.data(),
-                           SIGNAL(statusChanged(Tp::Connection::Status, Tp::ConnectionStatusReason)),
+                           SIGNAL(statusChanged(Tp::ConnectionStatus, Tp::ConnectionStatusReason)),
                            this,
-                           SLOT(expectConnReady(Tp::Connection::Status, Tp::ConnectionStatusReason))));
-        QCOMPARE(mConn->status(), Connection::StatusConnected);
+                           SLOT(expectConnReady(Tp::ConnectionStatus, Tp::ConnectionStatusReason))));
+        QCOMPARE(mConn->status(), ConnectionStatusConnected);
     }
 }
 
@@ -172,7 +177,7 @@ void TestHandles::testRequestAndRelease()
         << QLatin1String("bob") << QLatin1String("chris");
 
     // Request handles for the identifiers and wait for the request to process
-    PendingHandles *pending = mConn->requestHandles(Tp::HandleTypeContact, ids);
+    PendingHandles *pending = mConn->lowlevel()->requestHandles(Tp::HandleTypeContact, ids);
     QVERIFY(connect(pending,
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectPendingHandlesFinished(Tp::PendingOperation*))));
@@ -202,12 +207,6 @@ void TestHandles::testRequestAndRelease()
     handles = ReferencedHandles();
     mLoop->processEvents();
     processDBusQueue(mConn.data());
-
-    // Check that the handles have been released
-    for (int i = 0; i < 3; i++) {
-        uint handle = saveHandles[0];
-        QVERIFY(!tp_handle_is_valid(serviceRepo, handle, NULL));
-    }
 }
 
 void TestHandles::cleanup()
@@ -219,7 +218,7 @@ void TestHandles::cleanupTestCase()
 {
     if (mConn) {
         // Disconnect and wait for the readiness change
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
