@@ -2,7 +2,12 @@
 #include <QtCore/QPointer>
 #include <QtTest/QtTest>
 
+#define TP_QT4_ENABLE_LOWLEVEL_API
+
+#include <TelepathyQt4/ChannelFactory>
 #include <TelepathyQt4/Connection>
+#include <TelepathyQt4/ConnectionLowlevel>
+#include <TelepathyQt4/ContactFactory>
 #include <TelepathyQt4/ContactManager>
 #include <TelepathyQt4/PendingChannel>
 #include <TelepathyQt4/PendingContacts>
@@ -49,11 +54,13 @@ protected Q_SLOTS:
             const Tp::Contacts &,
             const Tp::Channel::GroupMemberChangeDetails &);
 
-    void onStreamRemoved(const Tp::MediaStreamPtr &);
-    void onStreamDirectionChanged(const Tp::MediaStreamPtr &,
+    void onStreamRemoved(const Tp::StreamedMediaStreamPtr &);
+    void onStreamDirectionChanged(const Tp::StreamedMediaStreamPtr &,
             Tp::MediaStreamDirection,
             Tp::MediaStreamPendingSend);
-    void onStreamStateChanged(const Tp::MediaStreamPtr &,
+    void onLSSChanged(Tp::StreamedMediaStream::SendingState);
+    void onRSSChanged(Tp::StreamedMediaStream::SendingState);
+    void onStreamStateChanged(const Tp::StreamedMediaStreamPtr &,
             Tp::MediaStreamState);
     void onChanInvalidated(Tp::DBusProxy *,
             const QString &, const QString &);
@@ -101,17 +108,19 @@ private:
     QString mConnPath;
     StreamedMediaChannelPtr mChan;
     QList<ContactPtr> mRequestContactsReturn;
-    MediaStreams mRequestStreamsReturn;
+    StreamedMediaStreams mRequestStreamsReturn;
     Contacts mChangedCurrent;
     Contacts mChangedLP;
     Contacts mChangedRP;
     Contacts mChangedRemoved;
     Channel::GroupMemberChangeDetails mDetails;
-    MediaStreamPtr mStreamRemovedReturn;
-    MediaStreamPtr mSDCStreamReturn;
+    StreamedMediaStreamPtr mStreamRemovedReturn;
+    StreamedMediaStreamPtr mSDCStreamReturn;
     Tp::MediaStreamDirection mSDCDirectionReturn;
     Tp::MediaStreamPendingSend mSDCPendingReturn;
-    MediaStreamPtr mSSCStreamReturn;
+    StreamedMediaStreamPtr mSSCStreamReturn;
+    Tp::StreamedMediaStream::SendingState mChangedLSS;
+    Tp::StreamedMediaStream::SendingState mChangedRSS;
     Tp::MediaStreamState mSSCStateReturn;
     QQueue<uint> mLocalHoldStates;
     QQueue<uint> mLocalHoldStateReasons;
@@ -217,7 +226,7 @@ void TestStreamedMediaChan::expectRequestStreamsFinished(PendingOperation *op)
 
     qDebug() << "request streams finished successfully";
 
-    PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
+    PendingStreamedMediaStreams *pms = qobject_cast<PendingStreamedMediaStreams*>(op);
     mRequestStreamsReturn = pms->streams();
     mLoop->exit(0);
 }
@@ -242,7 +251,7 @@ void TestStreamedMediaChan::expectBusyRequestStreamsFinished(PendingOperation *o
 
     qDebug() << "request streams finished successfully";
 
-    PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
+    PendingStreamedMediaStreams *pms = qobject_cast<PendingStreamedMediaStreams*>(op);
     mRequestStreamsReturn = pms->streams();
     mLoop->exit(0);
 }
@@ -253,14 +262,14 @@ void TestStreamedMediaChan::expectOutgoingRequestStreamsFinished(PendingOperatio
     QVERIFY(!op->isError());
     QVERIFY(op->isValid());
 
-    PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
+    PendingStreamedMediaStreams *pms = qobject_cast<PendingStreamedMediaStreams*>(op);
     mRequestStreamsReturn = pms->streams();
 
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
 
     QCOMPARE(mRequestContactsReturn.size(), 1);
-    MediaStreamPtr stream = mRequestStreamsReturn.first();
+    StreamedMediaStreamPtr stream = mRequestStreamsReturn.first();
     QCOMPARE(stream->contact(), otherContact);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeAudio);
 
@@ -361,14 +370,14 @@ void TestStreamedMediaChan::onGroupMembersChanged(
     mDetails = details;
 }
 
-void TestStreamedMediaChan::onStreamRemoved(const MediaStreamPtr &stream)
+void TestStreamedMediaChan::onStreamRemoved(const StreamedMediaStreamPtr &stream)
 {
     qDebug() << "stream" << stream.data() << "removed";
     mStreamRemovedReturn = stream;
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::onStreamDirectionChanged(const MediaStreamPtr &stream,
+void TestStreamedMediaChan::onStreamDirectionChanged(const StreamedMediaStreamPtr &stream,
         Tp::MediaStreamDirection direction,
         Tp::MediaStreamPendingSend pendingSend)
 {
@@ -379,7 +388,21 @@ void TestStreamedMediaChan::onStreamDirectionChanged(const MediaStreamPtr &strea
     mLoop->exit(0);
 }
 
-void TestStreamedMediaChan::onStreamStateChanged(const MediaStreamPtr &stream,
+void TestStreamedMediaChan::onLSSChanged(Tp::StreamedMediaStream::SendingState state)
+{
+    qDebug() << "onLSSChanged: " << static_cast<uint>(state);
+    mChangedLSS = state;
+    mLoop->exit(0);
+}
+
+void TestStreamedMediaChan::onRSSChanged(Tp::StreamedMediaStream::SendingState state)
+{
+    qDebug() << "onRSSChanged: " << static_cast<uint>(state);
+    mChangedRSS = state;
+    mLoop->exit(0);
+}
+
+void TestStreamedMediaChan::onStreamStateChanged(const StreamedMediaStreamPtr &stream,
         Tp::MediaStreamState state)
 {
     qDebug() << "stream" << stream.data() << "state changed to" << state;
@@ -411,7 +434,7 @@ void TestStreamedMediaChan::expectTerminateRequestStreamsFinished(PendingOperati
 
     QVERIFY(op->isValid());
 
-    PendingMediaStreams *pms = qobject_cast<PendingMediaStreams*>(op);
+    PendingStreamedMediaStreams *pms = qobject_cast<PendingStreamedMediaStreams*>(op);
     mRequestStreamsReturn = pms->streams();
 
     qDebug() << "stream requested successfully";
@@ -557,16 +580,18 @@ void TestStreamedMediaChan::initTestCase()
     g_free(name);
     g_free(connPath);
 
-    mConn = Connection::create(mConnName, mConnPath);
+    mConn = Connection::create(mConnName, mConnPath,
+            ChannelFactory::create(QDBusConnection::sessionBus()),
+            ContactFactory::create());
     QCOMPARE(mConn->isReady(), false);
 
-    QVERIFY(connect(mConn->requestConnect(Connection::FeatureSelfContact),
+    QVERIFY(connect(mConn->lowlevel()->requestConnect(Connection::FeatureSelfContact),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mConn->isReady(), true);
     QCOMPARE(static_cast<uint>(mConn->status()),
-             static_cast<uint>(Connection::StatusConnected));
+             static_cast<uint>(ConnectionStatusConnected));
 }
 
 void TestStreamedMediaChan::init()
@@ -584,6 +609,8 @@ void TestStreamedMediaChan::init()
     mSDCDirectionReturn = (Tp::MediaStreamDirection) -1;
     mSDCPendingReturn = (Tp::MediaStreamPendingSend) -1;
     mSSCStateReturn = (Tp::MediaStreamState) -1;
+    mChangedLSS = (Tp::StreamedMediaStream::SendingState) -1;
+    mChangedRSS = (Tp::StreamedMediaStream::SendingState) -1;
     mSSCStreamReturn.reset();
     mLocalHoldStates.clear();
     mLocalHoldStateReasons.clear();
@@ -594,7 +621,9 @@ void TestStreamedMediaChan::testOutgoingCall()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("alice")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
     QVERIFY(mRequestContactsReturn.size() == 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
@@ -606,7 +635,7 @@ void TestStreamedMediaChan::testOutgoingCall()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -686,7 +715,7 @@ void TestStreamedMediaChan::testOutgoingCall()
                     SLOT(expectRequestStreamsFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mRequestStreamsReturn.size(), 1);
-    MediaStreamPtr stream = mRequestStreamsReturn.first();
+    StreamedMediaStreamPtr stream = mRequestStreamsReturn.first();
     QCOMPARE(stream->contact(), otherContact);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeVideo);
 
@@ -709,8 +738,8 @@ void TestStreamedMediaChan::testOutgoingCall()
     qDebug() << "removing audio stream";
 
     QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamRemoved(const Tp::MediaStreamPtr &)),
-                    SLOT(onStreamRemoved(const Tp::MediaStreamPtr &))));
+                    SIGNAL(streamRemoved(const Tp::StreamedMediaStreamPtr &)),
+                    SLOT(onStreamRemoved(const Tp::StreamedMediaStreamPtr &))));
     QVERIFY(connect(mChan->removeStream(stream),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
@@ -734,9 +763,9 @@ void TestStreamedMediaChan::testOutgoingCall()
 
     if (stream->state() != Tp::MediaStreamStateConnected) {
         QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamStateChanged(const Tp::MediaStreamPtr &,
+                    SIGNAL(streamStateChanged(const Tp::StreamedMediaStreamPtr &,
                             Tp::MediaStreamState)),
-                    SLOT(onStreamStateChanged(const Tp::MediaStreamPtr &,
+                    SLOT(onStreamStateChanged(const Tp::StreamedMediaStreamPtr &,
                             Tp::MediaStreamState))));
     } else {
         // Pretend that we saw the SSC to Connected (although it might have happened even before the
@@ -746,28 +775,45 @@ void TestStreamedMediaChan::testOutgoingCall()
         mSSCStateReturn = Tp::MediaStreamStateConnected;
     }
 
+    QCOMPARE(stream->localSendingRequested(), false);
+    QCOMPARE(stream->remoteSendingRequested(), false);
+    QCOMPARE(stream->sending(), true);
+    QCOMPARE(stream->receiving(), true);
+
+    /* request only receiving now */
     QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamDirectionChanged(const Tp::MediaStreamPtr &,
+                    SIGNAL(streamDirectionChanged(const Tp::StreamedMediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
                                                   Tp::MediaStreamPendingSend)),
-                    SLOT(onStreamDirectionChanged(const Tp::MediaStreamPtr &,
+                    SLOT(onStreamDirectionChanged(const Tp::StreamedMediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
                                                   Tp::MediaStreamPendingSend))));
+    QVERIFY(connect(stream.data(),
+                    SIGNAL(localSendingStateChanged(Tp::StreamedMediaStream::SendingState)),
+                    SLOT(onLSSChanged(Tp::StreamedMediaStream::SendingState))));
+    QVERIFY(connect(stream.data(),
+                    SIGNAL(remoteSendingStateChanged(Tp::StreamedMediaStream::SendingState)),
+                    SLOT(onRSSChanged(Tp::StreamedMediaStream::SendingState))));
     QVERIFY(connect(stream->requestDirection(Tp::MediaStreamDirectionReceive),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    while (!mSDCStreamReturn || !mSSCStreamReturn) {
+    while (!mSDCStreamReturn || !mSSCStreamReturn || (static_cast<int>(mChangedLSS) == -1)) {
         qDebug() << "re-entering mainloop to wait for stream direction change and state change";
         // wait direction and state changed signal
         QCOMPARE(mLoop->exec(), 0);
     }
+    // If this fails, we also got a remote state change signal, although we shouldn't have
+    QCOMPARE(static_cast<int>(mChangedRSS), -1);
     QCOMPARE(mSDCStreamReturn, stream);
     QVERIFY(mSDCDirectionReturn & Tp::MediaStreamDirectionReceive);
     QVERIFY(stream->direction() & Tp::MediaStreamDirectionReceive);
     QCOMPARE(stream->pendingSend(), mSDCPendingReturn);
     QCOMPARE(mSSCStreamReturn, stream);
     QCOMPARE(mSSCStateReturn, Tp::MediaStreamStateConnected);
+
+    QCOMPARE(stream->sending(), false);
+    QCOMPARE(stream->receiving(), true);
 }
 
 void TestStreamedMediaChan::testOutgoingCallBusy()
@@ -777,7 +823,9 @@ void TestStreamedMediaChan::testOutgoingCallBusy()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("alice (busy)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
     QVERIFY(mRequestContactsReturn.size() == 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
@@ -789,7 +837,7 @@ void TestStreamedMediaChan::testOutgoingCallBusy()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -842,8 +890,10 @@ void TestStreamedMediaChan::testOutgoingCallNoAnswer()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("alice (no answer)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-    QVERIFY(mRequestContactsReturn.size() == 1);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
+    QCOMPARE(mRequestContactsReturn.size(), 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
 
@@ -854,7 +904,7 @@ void TestStreamedMediaChan::testOutgoingCallNoAnswer()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -926,8 +976,10 @@ void TestStreamedMediaChan::testOutgoingCallTerminate()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("alice (terminate)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
-    QVERIFY(mRequestContactsReturn.size() == 1);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
+    QCOMPARE(mRequestContactsReturn.size(), 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
 
@@ -938,7 +990,7 @@ void TestStreamedMediaChan::testOutgoingCallTerminate()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -1009,11 +1061,15 @@ void TestStreamedMediaChan::testOutgoingCallTerminate()
 
 void TestStreamedMediaChan::testIncomingCall()
 {
-    mConn->setSelfPresence(QLatin1String("away"), QLatin1String("preparing for a test"));
-    QVERIFY(connect(mConn->requestsInterface(),
+    mConn->lowlevel()->setSelfPresence(QLatin1String("away"), QLatin1String("preparing for a test"));
+
+    Client::ConnectionInterfaceRequestsInterface *connRequestsInterface =
+        mConn->optionalInterface<Client::ConnectionInterfaceRequestsInterface>();
+
+    QVERIFY(connect(connRequestsInterface,
                     SIGNAL(NewChannels(const Tp::ChannelDetailsList&)),
                     SLOT(onNewChannels(const Tp::ChannelDetailsList&))));
-    mConn->setSelfPresence(QLatin1String("available"), QLatin1String("call me?"));
+    mConn->lowlevel()->setSelfPresence(QLatin1String("available"), QLatin1String("call me?"));
     QCOMPARE(mLoop->exec(), 0);
 
     QVERIFY(mChan);
@@ -1047,7 +1103,7 @@ void TestStreamedMediaChan::testIncomingCall()
     QVERIFY(mChan->groupContacts().contains(mConn->selfContact()));
 
     QCOMPARE(mChan->streams().size(), 1);
-    MediaStreamPtr stream = mChan->streams().first();
+    StreamedMediaStreamPtr stream = mChan->streams().first();
     QCOMPARE(stream->channel(), mChan);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeAudio);
 
@@ -1091,9 +1147,9 @@ void TestStreamedMediaChan::testIncomingCall()
     qDebug() << "removing the audio stream";
 
     QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamRemoved(const Tp::MediaStreamPtr &)),
-                    SLOT(onStreamRemoved(const Tp::MediaStreamPtr &))));
-    QVERIFY(connect(mChan->removeStreams(MediaStreams() << stream),
+                    SIGNAL(streamRemoved(const Tp::StreamedMediaStreamPtr &)),
+                    SLOT(onStreamRemoved(const Tp::StreamedMediaStreamPtr &))));
+    QVERIFY(connect(mChan->removeStreams(StreamedMediaStreams() << stream),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -1115,9 +1171,9 @@ void TestStreamedMediaChan::testIncomingCall()
 
     if (stream->state() != Tp::MediaStreamStateConnected) {
         QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamStateChanged(const Tp::MediaStreamPtr &,
+                    SIGNAL(streamStateChanged(const Tp::StreamedMediaStreamPtr &,
                             Tp::MediaStreamState)),
-                    SLOT(onStreamStateChanged(const Tp::MediaStreamPtr &,
+                    SLOT(onStreamStateChanged(const Tp::StreamedMediaStreamPtr &,
                             Tp::MediaStreamState))));
     } else {
         // Pretend that we saw the SSC to Connected (although it might have happened even before the
@@ -1128,21 +1184,29 @@ void TestStreamedMediaChan::testIncomingCall()
     }
 
     QVERIFY(connect(mChan.data(),
-                    SIGNAL(streamDirectionChanged(const Tp::MediaStreamPtr &,
+                    SIGNAL(streamDirectionChanged(const Tp::StreamedMediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
                                                   Tp::MediaStreamPendingSend)),
-                    SLOT(onStreamDirectionChanged(const Tp::MediaStreamPtr &,
+                    SLOT(onStreamDirectionChanged(const Tp::StreamedMediaStreamPtr &,
                                                   Tp::MediaStreamDirection,
                                                   Tp::MediaStreamPendingSend))));
+    QVERIFY(connect(stream.data(),
+                    SIGNAL(localSendingStateChanged(Tp::StreamedMediaStream::SendingState)),
+                    SLOT(onLSSChanged(Tp::StreamedMediaStream::SendingState))));
+    QVERIFY(connect(stream.data(),
+                    SIGNAL(remoteSendingStateChanged(Tp::StreamedMediaStream::SendingState)),
+                    SLOT(onRSSChanged(Tp::StreamedMediaStream::SendingState))));
     QVERIFY(connect(stream->requestDirection(false, true),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
-    while (!mSDCStreamReturn || !mSSCStreamReturn) {
+    while (!mSDCStreamReturn || !mSSCStreamReturn || (static_cast<int>(mChangedLSS) == -1)) {
         // wait direction and state changed signal
         qDebug() << "re-entering mainloop to wait for stream direction change and state change";
         QCOMPARE(mLoop->exec(), 0);
     }
+    // If this fails, we also got a remote state change signal, although we shouldn't have
+    QCOMPARE(static_cast<int>(mChangedRSS), -1);
     QCOMPARE(mSDCStreamReturn, stream);
     QVERIFY(mSDCDirectionReturn & Tp::MediaStreamDirectionReceive);
     QVERIFY(stream->direction() & Tp::MediaStreamDirectionReceive);
@@ -1156,7 +1220,9 @@ void TestStreamedMediaChan::testHold()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("bob")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
     QVERIFY(mRequestContactsReturn.size() == 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
@@ -1168,7 +1234,7 @@ void TestStreamedMediaChan::testHold()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -1225,7 +1291,9 @@ void TestStreamedMediaChan::testHoldNoUnhold()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("bob (no unhold)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
     QVERIFY(mRequestContactsReturn.size() == 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
@@ -1237,7 +1305,7 @@ void TestStreamedMediaChan::testHoldNoUnhold()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -1289,7 +1357,9 @@ void TestStreamedMediaChan::testHoldInabilityUnhold()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("bob (inability to unhold)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
     QVERIFY(mRequestContactsReturn.size() == 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
@@ -1301,7 +1371,7 @@ void TestStreamedMediaChan::testHoldInabilityUnhold()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -1371,7 +1441,9 @@ void TestStreamedMediaChan::testDTMF()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("john")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
     QVERIFY(mRequestContactsReturn.size() == 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
@@ -1383,7 +1455,7 @@ void TestStreamedMediaChan::testDTMF()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -1402,7 +1474,7 @@ void TestStreamedMediaChan::testDTMF()
                     SLOT(expectRequestStreamsFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mRequestStreamsReturn.size(), 1);
-    MediaStreamPtr stream = mRequestStreamsReturn.first();
+    StreamedMediaStreamPtr stream = mRequestStreamsReturn.first();
     QCOMPARE(stream->contact(), otherContact);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeAudio);
 
@@ -1458,7 +1530,9 @@ void TestStreamedMediaChan::testDTMFNoContinuousTone()
     QVERIFY(connect(mConn->contactManager()->contactsForIdentifiers(QStringList() << QLatin1String("john (no continuous tone)")),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectRequestContactsFinished(Tp::PendingOperation*))));
-    QCOMPARE(mLoop->exec(), 0);
+    while (mRequestContactsReturn.size() != 1) {
+        QCOMPARE(mLoop->exec(), 0);
+    }
     QVERIFY(mRequestContactsReturn.size() == 1);
     ContactPtr otherContact = mRequestContactsReturn.first();
     QVERIFY(otherContact);
@@ -1470,7 +1544,7 @@ void TestStreamedMediaChan::testDTMFNoContinuousTone()
                    (uint) Tp::HandleTypeContact);
     request.insert(QLatin1String(TELEPATHY_INTERFACE_CHANNEL ".TargetHandle"),
                    otherContact->handle()[0]);
-    QVERIFY(connect(mConn->createChannel(request),
+    QVERIFY(connect(mConn->lowlevel()->createChannel(request),
                     SIGNAL(finished(Tp::PendingOperation*)),
                     SLOT(expectCreateChannelFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
@@ -1489,7 +1563,7 @@ void TestStreamedMediaChan::testDTMFNoContinuousTone()
                     SLOT(expectRequestStreamsFinished(Tp::PendingOperation*))));
     QCOMPARE(mLoop->exec(), 0);
     QCOMPARE(mRequestStreamsReturn.size(), 1);
-    MediaStreamPtr stream = mRequestStreamsReturn.first();
+    StreamedMediaStreamPtr stream = mRequestStreamsReturn.first();
     QCOMPARE(stream->contact(), otherContact);
     QCOMPARE(stream->type(), Tp::MediaStreamTypeAudio);
 
@@ -1528,7 +1602,7 @@ void TestStreamedMediaChan::cleanup()
 void TestStreamedMediaChan::cleanupTestCase()
 {
     if (mConn) {
-        QVERIFY(connect(mConn->requestDisconnect(),
+        QVERIFY(connect(mConn->lowlevel()->requestDisconnect(),
                         SIGNAL(finished(Tp::PendingOperation*)),
                         SLOT(expectSuccessfulCall(Tp::PendingOperation*))));
         QCOMPARE(mLoop->exec(), 0);
